@@ -14,6 +14,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "DataNode.h"
 #include "DataWriter.h"
+#include "Format.h"
 
 #include <algorithm>
 #include <sstream>
@@ -21,6 +22,8 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
+	// For tracking the player's average income, store daily net worth over this
+	// number of days.
 	static const unsigned HISTORY = 100;
 }
 
@@ -34,13 +37,14 @@ Account::Account()
 
 
 
-// Load or save account data.
+// Load account information from a data file (saved game or starting conditions).
 void Account::Load(const DataNode &node)
 {
 	credits = 0;
 	salariesOwed = 0;
 	creditScore = 400;
 	history.clear();
+	mortgages.clear();
 	
 	for(const DataNode &child : node)
 	{
@@ -63,6 +67,7 @@ void Account::Load(const DataNode &node)
 
 
 
+// Write account information to a saved game file.
 void Account::Save(DataWriter &out) const
 {
 	out.Write("account");
@@ -89,7 +94,7 @@ void Account::Save(DataWriter &out) const
 
 
 
-// Get or change the player's credits.
+// How much the player currently has in the bank.
 int64_t Account::Credits() const
 {
 	return credits;
@@ -97,6 +102,8 @@ int64_t Account::Credits() const
 
 
 
+// Give the player credits (or pass  negative number to subtract). If subtracting,
+// the calling function needs to check that this will not result in negative credits.
 void Account::AddCredits(int64_t value)
 {
 	credits += value;
@@ -104,6 +111,7 @@ void Account::AddCredits(int64_t value)
 
 
 
+// Pay down extra principal on a mortgage.
 void Account::PayExtra(int mortgage, int64_t amount)
 {
 	if(static_cast<unsigned>(mortgage) >= mortgages.size() || amount > credits
@@ -113,6 +121,8 @@ void Account::PayExtra(int mortgage, int64_t amount)
 	mortgages[mortgage].PayExtra(amount);
 	credits -= amount;
 	
+	// If this payment was for the entire remaining amount in the mortgage,
+	// remove it from the list.
 	if(!mortgages[mortgage].Principal())
 		mortgages.erase(mortgages.begin() + mortgage);
 }
@@ -124,16 +134,19 @@ string Account::Step(int64_t assets, int64_t salaries)
 {
 	ostringstream out;
 	
+	// Keep track of what payments were made and whether any could not be made.
 	salariesOwed += salaries;
-	bool hasDebts = !mortgages.empty() || salariesOwed;
 	bool paid = true;
 	
+	// Crew salaries take highest priority.
 	int64_t salariesPaid = salariesOwed;
 	if(salariesOwed)
 	{
 		if(salariesOwed > credits)
 		{
-			salariesPaid = max(credits, static_cast<int64_t>(0));
+			// If you can't pay the full salary amount, still pay some of it and
+			// remember how much back wages you owe to your crew.
+			salariesPaid = max<int64_t>(credits, 0);
 			salariesOwed -= salariesPaid;
 			credits -= salariesPaid;
 			paid = false;
@@ -146,6 +159,8 @@ string Account::Step(int64_t assets, int64_t salaries)
 		}
 	}
 	
+	// Unlike salaries, each mortgage payment must either be made in its entirety,
+	// or skipped completely (accruing interest and reducing your credit score).
 	int64_t mortgagesPaid = 0;
 	int64_t finesPaid = 0;
 	for(Mortgage &mortgage : mortgages)
@@ -162,6 +177,7 @@ string Account::Step(int64_t assets, int64_t salaries)
 		{
 			payment = mortgage.MakePayment();
 			credits -= payment;
+			// For the status text, keep track of whether this is a mortgage or a fine.
 			if(mortgage.Type() == "Mortgage")
 				mortgagesPaid += payment;
 			else
@@ -169,6 +185,7 @@ string Account::Step(int64_t assets, int64_t salaries)
 		}
 		assets -= mortgage.Principal();
 	}
+	// If any mortgage has been fully paid off, remove it from the list.
 	for(auto it = mortgages.begin(); it != mortgages.end(); )
 	{
 		if(!it->Principal())
@@ -180,14 +197,14 @@ string Account::Step(int64_t assets, int64_t salaries)
 	// Keep track of your net worth over the last HISTORY days.
 	if(history.size() > HISTORY)
 		history.erase(history.begin());
-	history.push_back(credits + assets);
+	history.push_back(credits + assets - salariesOwed);
 	
-	if(hasDebts)
-	{
-		creditScore += paid ? 1 : -5;
-		creditScore = max(200, min(800, creditScore));
-	}
+	// If you failed to pay any debt, your credit score drops. Otherwise, even
+	// if you have no debts, it increases. (Because, having no debts at all
+	// makes you at least as credit-worthy as someone who pays debts on time.)
+	creditScore = max(200, min(800, creditScore + (paid ? 1 : -5)));
 	
+	// If you didn't make any payments, no need to continue further.
 	if(!(salariesPaid + mortgagesPaid + finesPaid))
 		return out.str();
 	
@@ -196,26 +213,42 @@ string Account::Step(int64_t assets, int64_t salaries)
 	// If you made payments of all three types, the punctuation needs to
 	// include commas, so just handle that separately here.
 	if(salariesPaid && mortgagesPaid && finesPaid)
-		out << salariesPaid << " credits in crew salaries, " << mortgagesPaid
-			<< " in mortgages, and " << finesPaid << " in other payments.";
+		out << Format::Number(salariesPaid) << " credits in crew salaries, " << Format::Number(mortgagesPaid)
+			<< " in mortgages, and " << Format::Number(finesPaid) << " in fines.";
 	else
 	{
 		if(salariesPaid)
-			out << salariesPaid << ((mortgagesPaid || finesPaid) ?
+			out << Format::Number(salariesPaid) << ((mortgagesPaid || finesPaid) ?
 				" credits in crew salaries and " : " credits in crew salaries.");
 		if(mortgagesPaid)
-			out << mortgagesPaid << (salariesPaid ? " " : " credits ")
+			out << Format::Number(mortgagesPaid) << (salariesPaid ? " " : " credits ")
 				<< (finesPaid ? "in mortgage payments and " : "in mortgage payments.");
 		if(finesPaid)
-			out << finesPaid << ((salariesPaid || mortgagesPaid) ?
-				" in other payments." : " credits in other payments.");
+			out << Format::Number(finesPaid) << ((salariesPaid || mortgagesPaid) ?
+				" in fines." : " credits in fines.");
 	}
 	return out.str();
 }
 
 
 
-// Liabilities:
+int64_t Account::SalariesOwed() const
+{
+	return salariesOwed;
+}
+
+
+
+void Account::PaySalaries(int64_t amount)
+{
+	amount = min(min(amount, salariesOwed), credits);
+	credits -= amount;
+	salariesOwed -= amount;
+}
+
+
+
+// Access the list of mortgages.
 const vector<Mortgage> &Account::Mortgages() const
 {
 	return mortgages;
@@ -223,6 +256,8 @@ const vector<Mortgage> &Account::Mortgages() const
 
 
 
+// Add a new mortgage for the given amount, with an interest rate determined by
+// your credit score.
 void Account::AddMortgage(int64_t principal)
 {
 	mortgages.emplace_back(principal, creditScore);
@@ -231,6 +266,7 @@ void Account::AddMortgage(int64_t principal)
 
 
 
+// Add a "fine" with a high, fixed interest rate and a short term.
 void Account::AddFine(int64_t amount)
 {
 	mortgages.emplace_back(amount, 0, 60);
@@ -238,24 +274,28 @@ void Account::AddFine(int64_t amount)
 
 
 
-void Account::AddBonus(int64_t bonus)
-{
-	mortgages.emplace_back(bonus, 1000, 60);
-}
-
-
-
+// Check how big a mortgage the player can afford to pay at their current income.
 int64_t Account::Prequalify() const
 {
 	int64_t payments = 0;
+	int64_t liabilities = 0;
 	for(const Mortgage &mortgage : mortgages)
+	{
 		payments += mortgage.Payment();
-	return Mortgage::Maximum(YearlyRevenue(), creditScore, payments);
+		liabilities += mortgage.Principal();
+	}
+	
+	// Put a limit on new debt that the player can take out, as a fraction of
+	// their net worth, to avoid absurd mortgages being offered when the player
+	// has just captured some very lucrative ships.
+	return max<int64_t>(0, min(
+		NetWorth() / 3 + 500000 - liabilities,
+		Mortgage::Maximum(YearlyRevenue(), creditScore, payments)));
 }
 
 
 
-// Assets:
+// Get the player's total net worth (counting all ships and all debts).
 int64_t Account::NetWorth() const
 {
 	return history.empty() ? 0 : history.back();
@@ -263,25 +303,22 @@ int64_t Account::NetWorth() const
 
 
 
-const vector<int64_t> Account::History() const
-{
-	return history;
-}
-
-
-
-int64_t Account::YearlyRevenue() const
-{
-	if(history.empty() || history.back() <= history.front())
-		return 0;
-	
-	return ((history.back() - history.front()) * 365) / HISTORY;
-}
-
-
-
 // Find out the player's credit rating.
 int Account::CreditScore() const
 {
 	return creditScore;
 }
+
+
+
+// Extrapolate from the player's current net worth history to determine how much
+// their net worth is expected to change over the course of the next year.
+int64_t Account::YearlyRevenue() const
+{
+	if(history.empty() || history.back() <= history.front())
+		return 0;
+	
+	// Note that this intentionally under-estimates if the player has not yet
+	// played for long enough to accumulate a full income history.
+	return ((history.back() - history.front()) * 365) / HISTORY;
+}
