@@ -13,20 +13,23 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "BoardingPanel.h"
 
 #include "CargoHold.h"
+#include "Depreciation.h"
+#include "Dialog.h"
 #include "FillShader.h"
 #include "Font.h"
 #include "FontSet.h"
 #include "Format.h"
 #include "GameData.h"
 #include "Government.h"
-#include "InfoPanel.h"
 #include "Information.h"
 #include "Interface.h"
 #include "Messages.h"
 #include "PlayerInfo.h"
+#include "Preferences.h"
 #include "Random.h"
 #include "Ship.h"
 #include "ShipEvent.h"
+#include "ShipInfoPanel.h"
 #include "System.h"
 #include "UI.h"
 
@@ -49,30 +52,69 @@ namespace {
 
 
 
+// Constructor.
 BoardingPanel::BoardingPanel(PlayerInfo &player, const shared_ptr<Ship> &victim)
 	: player(player), you(player.FlagshipPtr()), victim(victim),
-	attackOdds(&*you, &*victim), defenseOdds(&*victim, &*you),
-	initialCrew(you->Crew())
+	attackOdds(*you, *victim), defenseOdds(*victim, *you)
 {
+	// The escape key should close this panel rather than bringing up the main menu.
 	SetInterruptible(false);
 	
+	// Figure out how much the victim's commodities are worth in the current
+	// system and add them to the list of plunder.
 	const System &system = *player.GetSystem();
 	for(const auto &it : victim->Cargo().Commodities())
-		plunder.emplace_back(it.first, it.second, system.Trade(it.first));
+		if(it.second)
+			plunder.emplace_back(it.first, it.second, system.Trade(it.first));
 	
 	// You cannot plunder hand to hand weapons, because they are kept in the
-	// crew's quarters, not mounted on the exterior of the ship.
-	for(const auto &it : victim->Outfits())
-		if(it.first->Category() != "Hand to Hand")
-			plunder.emplace_back(it.first, it.second);
+	// crew's quarters, not mounted on the exterior of the ship. Certain other
+	// outfits are also unplunderable, like mass expansions.
+	auto sit = victim->Outfits().begin();
+	auto cit = victim->Cargo().Outfits().begin();
+	while(sit != victim->Outfits().end() || cit != victim->Cargo().Outfits().end())
+	{
+		const Outfit *outfit = nullptr;
+		int count = 0;
+		// Merge the outfit lists from the ship itself and its cargo bay. If an
+		// outfit exists in both locations, combine the counts.
+		bool shipIsFirst = (cit == victim->Cargo().Outfits().end() || 
+			(sit != victim->Outfits().end() && sit->first <= cit->first));
+		bool cargoIsFirst = (sit == victim->Outfits().end() ||
+			(cit != victim->Cargo().Outfits().end() && cit->first <= sit->first));
+		if(shipIsFirst)
+		{
+			outfit = sit->first;
+			// Don't include outfits that are installed and unplunderable. But,
+			// "unplunderable" outfits can still be stolen from cargo.
+			if(!sit->first->Get("unplunderable"))
+				count += sit->second;
+			++sit;
+		}
+		if(cargoIsFirst)
+		{
+			outfit = cit->first;
+			count += cit->second;
+			++cit;
+		}
+		if(outfit && count)
+			plunder.emplace_back(outfit, count);
+	}
 	
+	// Some "ships" do not represent something the player could actually pilot.
+	if(!victim->IsCapturable())
+		messages.emplace_back("This is not a ship that you can capture.");
+	
+	// Sort the plunder by price per ton.
 	sort(plunder.begin(), plunder.end());
 }
 
 
-	
-void BoardingPanel::Draw() const
+
+// Draw the panel.
+void BoardingPanel::Draw()
 {
+	// Draw a translucent black scrim over everything beneath this panel.
 	DrawBackdrop();
 	
 	// Draw the list of plunder.
@@ -88,17 +130,19 @@ void BoardingPanel::Draw() const
 	int endY = 60;
 	
 	const Font &font = FontSet::Get(14);
+	// Y offset to center the text in a 20-pixel high row.
 	double fontOff = .5 * (20 - font.Height());
-	int freeSpace = you->Cargo().Free();
 	for( ; y < endY && static_cast<unsigned>(index) < plunder.size(); y += 20, ++index)
 	{
 		const Plunder &item = plunder[index];
 		
+		// Check if this is the selected row.
 		bool isSelected = (index == selected);
 		if(isSelected)
 			FillShader::Fill(Point(-155., y + 10.), Point(360., 20.), back);
 		
-		const Color &color = item.CanTake(freeSpace) ? isSelected ? bright : medium : dim;
+		// Color the item based on whether you have space for it.
+		const Color &color = item.CanTake(*you) ? isSelected ? bright : medium : dim;
 		Point pos(-320., y + fontOff);
 		font.Draw(item.Name(), pos, color);
 		
@@ -109,6 +153,7 @@ void BoardingPanel::Draw() const
 		font.Draw(item.Size(), sizePos, color);
 	}
 	
+	// Set which buttons are active.
 	Information info;
 	if(CanExit())
 		info.SetCondition("can exit");
@@ -116,7 +161,7 @@ void BoardingPanel::Draw() const
 		info.SetCondition("can take");
 	if(CanCapture())
 		info.SetCondition("can capture");
-	if(CanAttack() && you->Crew() > 1)
+	if(CanAttack() && (you->Crew() > 1 || !victim->RequiredCrew()))
 		info.SetCondition("can attack");
 	if(CanAttack())
 		info.SetCondition("can defend");
@@ -125,34 +170,45 @@ void BoardingPanel::Draw() const
 	int crew = 0;
 	if(you)
 	{
-		const Ship &ship = *you;
-		crew = ship.Crew();
-		info.SetString("cargo space", to_string(freeSpace));
+		crew = you->Crew();
+		info.SetString("cargo space", to_string(you->Cargo().Free()));
 		info.SetString("your crew", to_string(crew));
 		info.SetString("your attack",
 			Round(attackOdds.AttackerPower(crew)));
 		info.SetString("your defense",
 			Round(defenseOdds.DefenderPower(crew)));
 	}
-	int vCrew = victim->Crew();
-	info.SetString("enemy crew", to_string(vCrew));
-	info.SetString("enemy attack",
-		Round(defenseOdds.AttackerPower(vCrew)));
-	info.SetString("enemy defense",
-		Round(attackOdds.DefenderPower(vCrew)));
-	
-	info.SetString("attack odds",
-		Round(100. * attackOdds.Odds(crew, vCrew)) + "%");
-	info.SetString("attack casualties",
-		Round(attackOdds.AttackerCasualties(crew, vCrew)));
-	info.SetString("defense odds",
-		Round(100. * (1. - defenseOdds.Odds(vCrew, crew))) + "%");
-	info.SetString("defense casualties",
-		Round(defenseOdds.DefenderCasualties(vCrew, crew)));
+	int vCrew = victim ? victim->Crew() : 0;
+	if(victim && (victim->IsCapturable() || victim->GetGovernment()->IsPlayer()))
+	{
+		info.SetString("enemy crew", to_string(vCrew));
+		info.SetString("enemy attack",
+			Round(defenseOdds.AttackerPower(vCrew)));
+		info.SetString("enemy defense",
+			Round(attackOdds.DefenderPower(vCrew)));
+	}
+	if(victim && victim->IsCapturable() && !victim->GetGovernment()->IsPlayer())
+	{
+		// If you haven't initiated capture yet, show the self destruct odds in
+		// the attack odds. It's illogical for you to have access to that info,
+		// but not knowing what your true odds are is annoying.
+		double odds = attackOdds.Odds(crew, vCrew);
+		if(!isCapturing)
+			odds *= (1. - victim->Attributes().Get("self destruct"));
+		info.SetString("attack odds",
+			Round(100. * odds) + "%");
+		info.SetString("attack casualties",
+			Round(attackOdds.AttackerCasualties(crew, vCrew)));
+		info.SetString("defense odds",
+			Round(100. * (1. - defenseOdds.Odds(vCrew, crew))) + "%");
+		info.SetString("defense casualties",
+			Round(defenseOdds.DefenderCasualties(vCrew, crew)));
+	}
 	
 	const Interface *interface = GameData::Interfaces().Get("boarding");
-	interface->Draw(info);
+	interface->Draw(info, this);
 	
+	// Draw the status messages from hand to hand combat.
 	Point messagePos(50., 55.);
 	for(const string &message : messages)
 	{
@@ -163,20 +219,14 @@ void BoardingPanel::Draw() const
 
 
 
+// Handle key presses or button clicks that were mapped to key presses.
 bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 {
 	if((key == 'd' || key == 'x' || key == SDLK_ESCAPE || (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI)))) && CanExit())
 	{
+		// When closing the panel, mark the player dead if their ship was captured.
 		if(playerDied)
 			player.Die(true);
-		if(crewBonus)
-		{
-			Messages::Add(("You must pay " + Format::Number(crewBonus)
-				+ " credits in death benefits for the ")
-				+ ((casualties > 1) ? "families of your dead crew members."
-					: "family of your dead crew member."));
-			player.Accounts().AddBonus(crewBonus);
-		}
 		GetUI()->Pop(this);
 	}
 	else if(playerDied)
@@ -184,40 +234,47 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 	else if(key == 't' && CanTake())
 	{
 		CargoHold &cargo = you->Cargo();
-		int count = plunder[selected].CanTake(cargo.Free());
+		int count = plunder[selected].Count();
 		
 		const Outfit *outfit = plunder[selected].GetOutfit();
 		if(outfit)
 		{
 			// Check if this outfit is ammo for one of your weapons. If so, use
 			// it to refill your ammo rather than putting it in cargo.
-			int taken = 0;
+			int available = count;
+			// Keep track of how many you actually took.
+			count = 0;
 			for(const auto &it : you->Outfits())
 				if(it.first != outfit && it.first->Ammo() == outfit)
 				{
-					for( ; taken < count && you->Attributes().CanAdd(*outfit); ++taken)
-					{
-						you->AddOutfit(outfit, 1);
-						victim->AddOutfit(outfit, -1);
-					}
+					// Figure out how many of these outfits you can install.
+					count = you->Attributes().CanAdd(*outfit, available);
+					you->AddOutfit(outfit, count);
+					// You have now installed as many of these items as possible.
 					break;
 				}
-			cargo.Transfer(outfit, -(count - taken));
-			victim->AddOutfit(outfit, -(count - taken));
+			// Transfer as many as possible of these outfits to your cargo hold.
+			count += cargo.Add(outfit, available - count);
+			// Take outfits from cargo first, then from the ship itself.
+			int remaining = count - victim->Cargo().Remove(outfit, count);
+			victim->AddOutfit(outfit, -remaining);
 		}
 		else
-			victim->Cargo().Transfer(plunder[selected].Name(), count, &cargo);
+			count = victim->Cargo().Transfer(plunder[selected].Name(), count, &cargo);
 		
+		// If all of the plunder of this type was taken, remove it from the list.
+		// Otherwise, just update the count in the list item.
 		if(count == plunder[selected].Count())
 		{
 			plunder.erase(plunder.begin() + selected);
-			selected = min(selected, static_cast<int>(plunder.size()));
+			selected = min<int>(selected, plunder.size());
 		}
 		else
 			plunder[selected].Take(count);
 	}
 	else if((key == SDLK_UP || key == SDLK_DOWN || key == SDLK_PAGEUP || key == SDLK_PAGEDOWN) && !isCapturing)
 	{
+		// Scrolling the list of plunder.
 		if(key == SDLK_PAGEUP || key == SDLK_PAGEDOWN)
 			Drag(0, 200 * ((key == SDLK_PAGEDOWN) - (key == SDLK_PAGEUP)));
 		else
@@ -228,13 +285,23 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 				++selected;
 			
 			// Scroll down at least far enough to view the current item.
-			int minimumScroll = max(0, static_cast<int>(20 * selected - 200));
-			int maximumScroll = static_cast<int>(20 * selected);
+			double minimumScroll = max(0., 20. * selected - 200.);
+			double maximumScroll = 20. * selected;
 			scroll = max(minimumScroll, min(maximumScroll, scroll));
 		}
 	}
 	else if(key == 'c' && CanCapture())
 	{
+		// A ship that self-destructs checks once when you board it, and again
+		// when you try to capture it, to see if it will self-destruct. This is
+		// so that capturing will be harder than plundering.
+		if(Random::Real() < victim->Attributes().Get("self destruct"))
+		{
+			victim->SelfDestruct();
+			GetUI()->Pop(this);
+			GetUI()->Push(new Dialog("The moment you blast through the airlock, a series of explosions rocks the enemy ship. They appear to have set off their self-destruct sequence..."));
+			return true;
+		}
 		isCapturing = true;
 		messages.push_back("The airlock blasts open. Combat has begun!");
 		messages.push_back("(It will end if you both choose to \"defend.\")");
@@ -244,10 +311,17 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 		int yourStartCrew = you->Crew();
 		int enemyStartCrew = victim->Crew();
 		
-		// Figure out what action the other ship will take.
-		bool youAttack = (key == 'a' && yourStartCrew > 1);
+		// Figure out what action the other ship will take. As a special case,
+		// if you board them but immediately "defend" they will let you return
+		// to your ship in peace. That is to allow the player to "cancel" if
+		// they did not really mean to try to capture the ship.
+		bool youAttack = (key == 'a' && (yourStartCrew > 1 || !victim->RequiredCrew()));
 		bool enemyAttacks = defenseOdds.Odds(enemyStartCrew, yourStartCrew) > .5;
+		if(isFirstCaptureAction && !youAttack)
+			enemyAttacks = false;
+		isFirstCaptureAction = false;
 		
+		// If neither side attacks, combat ends.
 		if(!youAttack && !enemyAttacks)
 		{
 			messages.push_back("You retreat to your ships. Combat ends.");
@@ -260,6 +334,8 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			else if(enemyAttacks)
 				messages.push_back("You defend. ");
 			
+			// To speed things up, have multiple rounds of combat each time you
+			// click the button, if you started with a lot of crew.
 			int rounds = max(1, yourStartCrew / 5);
 			for(int round = 0; round < rounds; ++round)
 			{
@@ -268,21 +344,24 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 				if(!yourCrew || !enemyCrew)
 					break;
 				
-				unsigned yourPower = static_cast<unsigned>(1000. * (youAttack ?
-					attackOdds.AttackerPower(yourCrew) : defenseOdds.DefenderPower(yourCrew)));
-				unsigned enemyPower = static_cast<unsigned>(1000. * (enemyAttacks ?
-					defenseOdds.AttackerPower(enemyCrew) : attackOdds.DefenderPower(enemyCrew)));
+				// Your chance of winning this round is equal to the ratio of
+				// your power to the enemy's power.
+				double yourPower = (youAttack ?
+					attackOdds.AttackerPower(yourCrew) : defenseOdds.DefenderPower(yourCrew));
+				double enemyPower = (enemyAttacks ?
+					defenseOdds.AttackerPower(enemyCrew) : attackOdds.DefenderPower(enemyCrew));
 				
-				unsigned total = yourPower + enemyPower;
+				double total = yourPower + enemyPower;
 				if(!total)
 					break;
 				
-				if(Random::Int(total) >= yourPower)
+				if(Random::Real() * total >= yourPower)
 					you->AddCrew(-1);
 				else
 					victim->AddCrew(-1);
 			}
 			
+			// Report how many casualties each side suffered.
 			int yourCasualties = yourStartCrew - you->Crew();
 			int enemyCasualties = enemyStartCrew - victim->Crew();
 			if(yourCasualties && enemyCasualties)
@@ -293,6 +372,7 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			else if(enemyCasualties)
 				messages.back() += "They lose " + to_string(enemyCasualties) + " crew.";
 			
+			// Check if either ship has been captured.
 			if(!you->Crew())
 			{
 				messages.push_back("You have been killed. Your ship is lost.");
@@ -303,33 +383,28 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			}
 			else if(!victim->Crew())
 			{
-				casualties = initialCrew - you->Crew();
 				messages.push_back("You have succeeded in capturing this ship.");
+				victim->GetGovernment()->Offend(ShipEvent::CAPTURE, victim->RequiredCrew());
 				victim->WasCaptured(you);
 				if(!victim->JumpsRemaining() && you->CanRefuel(*victim))
-					you->TransferFuel(victim->JumpFuel(), &*victim);
+					you->TransferFuel(victim->JumpFuelMissing(), &*victim);
 				player.AddShip(victim);
-				if(!victim->CanBeCarried())
-					victim->SetParent(you);
-				else
-					for(const shared_ptr<Ship> &ship : player.Ships())
-						if(ship->CanHoldFighter(*victim))
-						{
-							victim->SetParent(ship);
-							break;
-						}
+				for(const Ship::Bay &bay : victim->Bays())
+					if(bay.ship)
+					{
+						player.AddShip(bay.ship);
+						player.HandleEvent(ShipEvent(you, bay.ship, ShipEvent::CAPTURE), GetUI());
+					}
 				isCapturing = false;
 				
-				int64_t bonus = (victim->Cost() * casualties) / (casualties + 2);
-				crewBonus += bonus;
-				
+				// Report this ship as captured in case any missions care.
 				ShipEvent event(you, victim, ShipEvent::CAPTURE);
 				player.HandleEvent(event, GetUI());
 			}
 		}
 	}
 	else if(command.Has(Command::INFO))
-		GetUI()->Push(new InfoPanel(player, true));
+		GetUI()->Push(new ShipInfoPanel(player));
 	
 	// Trim the list of status messages.
 	while(messages.size() > 5)
@@ -340,7 +415,8 @@ bool BoardingPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 
 
 
-bool BoardingPanel::Click(int x, int y)
+// Handle mouse clicks.
+bool BoardingPanel::Click(int x, int y, int clicks)
 {
 	// Was the click inside the plunder list?
 	if(x >= -330 && x < 20 && y >= -180 && y < 60)
@@ -351,39 +427,33 @@ bool BoardingPanel::Click(int x, int y)
 		return true;
 	}
 	
-	// Handle clicks on the interface buttons.
-	const Interface *interface = GameData::Interfaces().Get("boarding");
-	if(interface)
-	{
-		char key = interface->OnClick(Point(x, y));
-		if(key != '\0')
-			return DoKey(key);
-	}
-	
 	return true;
 }
 
 
 
-bool BoardingPanel::Drag(int dx, int dy)
+// Allow dragging of the plunder list.
+bool BoardingPanel::Drag(double dx, double dy)
 {
 	// The list is 240 pixels tall, and there are 10 pixels padding on the top
 	// and the bottom, so:
-	int maximumScroll = max(0, static_cast<int>(20 * plunder.size() - 220));
-	scroll = max(0, min(maximumScroll, scroll + dy));
+	double maximumScroll = max(0., 20. * plunder.size() - 220.);
+	scroll = max(0., min(maximumScroll, scroll - dy));
 	
 	return true;
 }
 
 
 
-bool BoardingPanel::Scroll(int dx, int dy)
+// The scroll wheel can be used to scroll the plunder list.
+bool BoardingPanel::Scroll(double dx, double dy)
 {
-	return Drag(dx, dy * -50);
+	return Drag(0., dy * Preferences::ScrollSpeed());
 }
 
 
 
+// You can't exit this panel if you're engaged in hand to hand combat.
 bool BoardingPanel::CanExit() const
 {
 	return !isCapturing;
@@ -391,7 +461,8 @@ bool BoardingPanel::CanExit() const
 
 
 
-bool BoardingPanel::CanTake(int index) const
+// Check if you can take the given plunder item.
+bool BoardingPanel::CanTake() const
 {
 	// If you ship or the other ship has been captured:
 	if(!you->GetGovernment()->IsPlayer())
@@ -400,17 +471,15 @@ bool BoardingPanel::CanTake(int index) const
 		return false;
 	if(isCapturing || playerDied)
 		return false;
-	
-	if(index < 0)
-		index = selected;
-	if(static_cast<unsigned>(index) >= plunder.size())
+	if(static_cast<unsigned>(selected) >= plunder.size())
 		return false;
 	
-	return plunder[index].CanTake(you->Cargo().Free());
+	return plunder[selected].CanTake(*you);
 }
 
 
 
+// Check if it's possible to initiate hand to hand combat.
 bool BoardingPanel::CanCapture() const
 {
 	// You can't click the "capture" button if you're already in combat mode.
@@ -422,27 +491,15 @@ bool BoardingPanel::CanCapture() const
 		return false;
 	if(victim->GetGovernment()->IsPlayer())
 		return false;
-	
-	if(victim->CanBeCarried())
-	{
-		// If this is an unpiloted drone, you don't need any crew to capture it.
-		// If it is a fighter you must have one crew member other than yourself.
-		if(you->Crew() < (victim->RequiredCrew() ? 2 : 1))
-			return false;
-		
-		// Check if any ship in your fleet can carry this ship.
-		for(const shared_ptr<Ship> &ship : player.Ships())
-			if(ship->CanHoldFighter(*victim))
-				return true;
-		
+	if(!victim->IsCapturable())
 		return false;
-	}
 	
-	return (you->Crew() > 1);
+	return (!victim->RequiredCrew() || you->Crew() > 1);
 }
 
 
 
+// Check if you are in the process of hand to hand combat.
 bool BoardingPanel::CanAttack() const
 {
 	return isCapturing;
@@ -450,6 +507,9 @@ bool BoardingPanel::CanAttack() const
 
 
 
+// Functions for BoardingPanel::Plunder:
+
+// Constructor (commodity cargo).
 BoardingPanel::Plunder::Plunder(const string &commodity, int count, int unitValue)
 	: name(commodity), outfit(nullptr), count(count), unitValue(unitValue)
 {
@@ -458,8 +518,9 @@ BoardingPanel::Plunder::Plunder(const string &commodity, int count, int unitValu
 
 
 
+// Constructor (outfit installed in the victim ship).
 BoardingPanel::Plunder::Plunder(const Outfit *outfit, int count)
-	: name(outfit->Name()), outfit(outfit), count(count), unitValue(outfit->Cost())
+	: name(outfit->Name()), outfit(outfit), count(count), unitValue(outfit->Cost() * Depreciation::Full())
 {
 	UpdateStrings();
 }
@@ -528,15 +589,21 @@ const Outfit *BoardingPanel::Plunder::GetOutfit() const
 
 // Find out how many of these I can take if I have this amount of cargo
 // space free.
-int BoardingPanel::Plunder::CanTake(int freeSpace) const
+bool BoardingPanel::Plunder::CanTake(const Ship &ship) const
 {
+	// If there's cargo space for this outfit, you can take it.
 	double mass = UnitMass();
-	if(mass <= 0.)
-		return count;
-	if(freeSpace <= 0)
-		return 0;
+	if(ship.Cargo().Free() >= mass)
+		return true;
 	
-	return min(count, static_cast<int>(freeSpace / mass));
+	// Otherwise, check if it is ammo for any of your weapons. If so, check if
+	// you can install it as an outfit.
+	if(outfit)
+		for(const auto &it : ship.Outfits())
+			if(it.first != outfit && it.first->Ammo() == outfit && ship.Attributes().CanAdd(*outfit))
+				return true;
+	
+	return false;
 }
 
 
@@ -550,21 +617,23 @@ void BoardingPanel::Plunder::Take(int count)
 
 
 
+// Update the text to reflect a change in the item count.
 void BoardingPanel::Plunder::UpdateStrings()
 {
-	int mass = static_cast<int>(UnitMass());
+	double mass = UnitMass();
 	if(!outfit)
 		size = to_string(count);
 	else if(count == 1)
-		size = to_string(mass);
+		size = Format::Number(mass);
 	else
-		size = to_string(count) + " x " + to_string(mass);
+		size = to_string(count) + " x " + Format::Number(mass);
 	
 	value = Format::Number(unitValue * count);
 }
 
 
 
+// Commodities come in units of one ton.
 double BoardingPanel::Plunder::UnitMass() const
 {
 	return outfit ? outfit->Get("mass") : 1.;
