@@ -18,6 +18,7 @@ using namespace std;
 
 
 
+// Constructor, taking a file path (in UTF-8).
 DataFile::DataFile(const string &path)
 {
 	Load(path);
@@ -25,6 +26,7 @@ DataFile::DataFile(const string &path)
 
 
 
+// Constructor, taking an istream. This can be cin or a file.
 DataFile::DataFile(istream &in)
 {
 	Load(in);
@@ -32,6 +34,7 @@ DataFile::DataFile(istream &in)
 
 
 
+// Load from a file path (in UTF-8).
 void DataFile::Load(const string &path)
 {
 	string data = Files::Read(path);
@@ -39,14 +42,19 @@ void DataFile::Load(const string &path)
 		return;
 	
 	// As a sentinel, make sure the file always ends in a newline.
-	if(data.back() != '\n')
+	if(data.empty() || data.back() != '\n')
 		data.push_back('\n');
 	
 	Load(&*data.begin(), &*data.end());
+	
+	// Note what file this node is in, so it will show up in error traces.
+	root.tokens.push_back("file");
+	root.tokens.push_back(path);
 }
 
 
 
+// Constructor, taking an istream. This can be cin or a file.
 void DataFile::Load(istream &in)
 {
 	vector<char> data;
@@ -68,6 +76,7 @@ void DataFile::Load(istream &in)
 
 
 
+// Get an iterator to the start of the list of nodes in this file.
 list<DataNode>::const_iterator DataFile::begin() const
 {
 	return root.begin();
@@ -75,6 +84,7 @@ list<DataNode>::const_iterator DataFile::begin() const
 
 
 
+// Get an iterator to the end of the list of nodes in this file.
 list<DataNode>::const_iterator DataFile::end() const
 {
 	return root.end();
@@ -82,8 +92,12 @@ list<DataNode>::const_iterator DataFile::end() const
 
 
 
+// Parse the given text.
 void DataFile::Load(const char *it, const char *end)
 {
+	// Keep track of the current stack of indentation levels and the most recent
+	// node at each level - that is, the node that will be the "parent" of any
+	// new node added at the next deeper indentation level.
 	vector<DataNode *> stack(1, &root);
 	vector<int> whiteStack(1, -1);
 	
@@ -124,6 +138,8 @@ void DataFile::Load(const char *it, const char *end)
 		// Tokenize the line. Skip comments and empty lines.
 		while(*it != '\n')
 		{
+			// Check if this token begins with a quotation mark. If so, it will
+			// include everything up to the next instance of that mark.
 			char endQuote = *it;
 			bool isQuoted = (endQuote == '"' || endQuote == '`');
 			it += isQuoted;
@@ -141,11 +157,14 @@ void DataFile::Load(const char *it, const char *end)
 				node.tokens.emplace_back();
 			else
 				node.tokens.emplace_back(start, it);
+			// This is not a fatal error, but it may indicate a format mistake:
 			if(isQuoted && *it == '\n')
 				node.PrintTrace("Closing quotation mark is missing:");
 			
 			if(*it != '\n')
 			{
+				// If we've not yet reached the end of the line of text, search
+				// forward for the next non-whitespace character.
 				it += isQuoted;
 				while(*it != '\n' && *it <= ' ' && *it != '#')
 					++it;
