@@ -15,6 +15,8 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Color.h"
 #include "Font.h"
 #include "FontSet.h"
+#include "GameData.h"
+#include "Government.h"
 #include "LineShader.h"
 #include "Point.h"
 #include "OutlineShader.h"
@@ -38,9 +40,9 @@ void EscortDisplay::Clear()
 
 
 
-void EscortDisplay::Add(const Ship &ship, bool isHere)
+void EscortDisplay::Add(const Ship &ship, bool isHere, bool fleetIsJumping, bool isSelected)
 {
-	icons.emplace_back(ship, isHere);
+	icons.emplace_back(ship, isHere, fleetIsJumping, isSelected);
 }
 
 
@@ -51,13 +53,19 @@ void EscortDisplay::Draw() const
 {
 	MergeStacks();
 	icons.sort();
+	stacks.clear();
+	zones.clear();
+	static const Set<Color> &colors = GameData::Colors();
 	
 	// Draw escort status.
-	static const Font &font = FontSet::Get(14);
+	const Font &font = FontSet::Get(14);
 	Point pos = Point(Screen::Left() + 20., Screen::Bottom());
-	static const Color hereColor(.8, 1.);
-	static const Color elsewhereColor(.4, .4, .6, 1.);
-	static const Color readyToJumpColor(.2, .8, .2, 1.);
+	const Color &elsewhereColor = *colors.Get("escort elsewhere");
+	const Color &cannotJumpColor = *colors.Get("escort blocked");
+	const Color &notReadyToJumpColor = *colors.Get("escort not ready");
+	const Color &selectedColor = *colors.Get("escort selected");
+	const Color &hereColor = *colors.Get("escort present");
+	const Color &hostileColor = *colors.Get("escort hostile");
 	for(const Icon &escort : icons)
 	{
 		if(!escort.sprite)
@@ -73,10 +81,16 @@ void EscortDisplay::Draw() const
 			font.Draw(escort.system, pos + Point(-10., 10.), elsewhereColor);
 
 		Color color;
-		if(!escort.isHere)
+		if(escort.isHostile)
+			color = hostileColor;
+		else if(!escort.isHere)
 			color = elsewhereColor;
-		else if(escort.isReadyToJump)
-			color = readyToJumpColor;
+		else if(escort.cannotJump)
+			color = cannotJumpColor;
+		else if(escort.notReadyToJump)
+			color = notReadyToJumpColor;
+		else if(escort.isSelected)
+			color = selectedColor;
 		else
 			color = hereColor;
 		
@@ -84,11 +98,13 @@ void EscortDisplay::Draw() const
 		double scale = min(20. / escort.sprite->Width(), 20. / escort.sprite->Height());
 		Point size(escort.sprite->Width() * scale, escort.sprite->Height() * scale);
 		OutlineShader::Draw(escort.sprite, pos, size, color);
+		zones.push_back(pos);
+		stacks.push_back(escort.ships);
 		// Draw the number of ships in this stack.
 		double width = 70.;
-		if(escort.stackSize > 1)
+		if(escort.ships.size() > 1)
 		{
-			string number = to_string(escort.stackSize);
+			string number = to_string(escort.ships.size());
 		
 			Point numberPos = pos;
 			numberPos.X() += 15. + width - font.Width(number);
@@ -99,12 +115,12 @@ void EscortDisplay::Draw() const
 		
 		// Draw the status bars.
 		static const Color fullColor[5] = {
-			Color(.44, .56, .70, 0), Color(.70, .62, .44, 0),
-			Color(.60, .60, .60, 0), Color(.70, .44, .44, 0), Color(.70, .62, .44, 0)
+			colors.Get("shields")->Additive(1.), colors.Get("hull")->Additive(1.),
+			colors.Get("energy")->Additive(1.), colors.Get("heat")->Additive(1.), colors.Get("fuel")->Additive(1.)
 		};
 		static const Color halfColor[5] = {
-			Color(.22, .28, .35, 0), Color(.35, .31, .22, 0),
-			Color(.30, .30, .30, 0), Color(.35, .22, .22, 0), Color(.35, .31, .22, 0)
+			fullColor[0].Additive(.5), fullColor[1].Additive(.5),
+			fullColor[2].Additive(.5), fullColor[3].Additive(.5), fullColor[4].Additive(.5),
 		};
 		Point from(pos.X() + 15., pos.Y() - 8.5);
 		for(int i = 0; i < 5; ++i)
@@ -137,15 +153,32 @@ void EscortDisplay::Draw() const
 
 
 
-EscortDisplay::Icon::Icon(const Ship &ship, bool isHere)
-	: sprite(ship.GetSprite().GetSprite()),
+// Check if the given point is a click on an escort icon. If so, return the
+// stack of ships represented by the icon. Otherwise, return an empty stack.
+const vector<const Ship *> &EscortDisplay::Click(const Point &point) const
+{
+	for(unsigned i = 0; i < zones.size(); ++i)
+		if(point.Distance(zones[i]) < 15.)
+			return stacks[i];
+	
+	static const vector<const Ship *> empty;
+	return empty;
+}
+
+
+
+EscortDisplay::Icon::Icon(const Ship &ship, bool isHere, bool fleetIsJumping, bool isSelected)
+	: sprite(ship.GetSprite()),
 	isHere(isHere && !ship.IsDisabled()),
-	isReadyToJump(ship.CheckHyperspace()),
-	stackSize(1),
+	isHostile(ship.GetGovernment() && ship.GetGovernment()->IsEnemy()),
+	notReadyToJump(fleetIsJumping && !ship.IsHyperspacing() && !ship.IsReadyToJump()),
+	cannotJump(fleetIsJumping && !ship.IsHyperspacing() && !ship.JumpsRemaining()),
+	isSelected(isSelected),
 	cost(ship.Cost()),
 	system((!isHere && ship.GetSystem()) ? ship.GetSystem()->Name() : ""),
 	low{ship.Shields(), ship.Hull(), ship.Energy(), ship.Heat(), ship.Fuel()},
-	high(low)
+	high(low),
+	ships(1, &ship)
 {
 }
 
@@ -169,8 +202,10 @@ int EscortDisplay::Icon::Height() const
 void EscortDisplay::Icon::Merge(const Icon &other)
 {
 	isHere &= other.isHere;
-	isReadyToJump &= other.isReadyToJump;
-	stackSize += other.stackSize;
+	isHostile |= other.isHostile;
+	notReadyToJump |= other.notReadyToJump;
+	cannotJump |= other.cannotJump;
+	isSelected |= other.isSelected;
 	if(system.empty() && !other.system.empty())
 		system = other.system;
 	
@@ -179,6 +214,7 @@ void EscortDisplay::Icon::Merge(const Icon &other)
 		low[i] = min(low[i], other.low[i]);
 		high[i] = max(high[i], other.high[i]);
 	}
+	ships.insert(ships.end(), other.ships.begin(), other.ships.end());
 }
 
 
@@ -197,7 +233,7 @@ void EscortDisplay::MergeStacks() const
 		int height = 0;
 		for(Icon &icon : icons)
 		{
-			if(unstackable.find(icon.sprite) == unstackable.end() && (!cheapest || *cheapest < icon))
+			if(!unstackable.count(icon.sprite) && (!cheapest || *cheapest < icon))
 				cheapest = &icon;
 			
 			height += icon.Height();
@@ -206,19 +242,36 @@ void EscortDisplay::MergeStacks() const
 		if(height < maxHeight || !cheapest)
 			break;
 		
-		// Merge all other instances of this ship's sprite with this icon.
+		// Merge together each group of escorts that have this icon and are in
+		// the same system and have the same attitude towards the player.
+		map<const bool, map<string, Icon *>> merged;
+		
+		// The "cheapest" element in the list may be removed to merge it with an
+		// earlier ship of the same type, so store a copy of its sprite pointer:
+		const Sprite *sprite = cheapest->sprite;
 		list<Icon>::iterator it = icons.begin();
 		while(it != icons.end())
 		{
-			if(&*it == cheapest || it->sprite != cheapest->sprite || it->isHere != cheapest->isHere)
+			if(it->sprite != sprite)
 			{
 				++it;
 				continue;
 			}
 			
-			cheapest->Merge(*it);
-			it = icons.erase(it);	
+			// If this is the first escort we've seen so far in its system, it
+			// is the one we will merge all others in this system into.
+			auto mit = merged[it->isHostile].find(it->system);
+			if(mit == merged[it->isHostile].end())
+			{
+				merged[it->isHostile][it->system] = &*it;
+				++it;
+			}
+			else
+			{
+				mit->second->Merge(*it);
+				it = icons.erase(it);	
+			}
 		}
-		unstackable.insert(cheapest->sprite);
+		unstackable.insert(sprite);
 	}
 }
