@@ -36,7 +36,9 @@ void GameEvent::Load(const DataNode &node)
 			date = Date(child.Value(1), child.Value(2), child.Value(3));
 		else if(child.Token(0) == "unvisit" && child.Size() >= 2)
 			systemsToUnvisit.push_back(GameData::Systems().Get(child.Token(1)));
-		else if(child.Token(0) == "system" || child.Token(0) == "planet"
+		else if(child.Token(0) == "unvisit planet" && child.Size() >= 2)
+			planetsToUnvisit.push_back(GameData::Planets().Get(child.Token(1)));
+		else if(child.Token(0) == "system" || child.Token(0) == "planet" || child.Token(0) == "galaxy"
 				|| child.Token(0) == "shipyard" || child.Token(0) == "outfitter"
 				|| child.Token(0) == "fleet" || child.Token(0) == "government"
 				|| child.Token(0) == "link" || child.Token(0) == "unlink")
@@ -60,6 +62,9 @@ void GameEvent::Save(DataWriter &out) const
 		for(const System *system : systemsToUnvisit)
 			if(system && !system->Name().empty())
 				out.Write("unvisit", system->Name());
+		for(const Planet *planet : planetsToUnvisit)
+			if(planet && !planet->Name().empty())
+				out.Write("unvisit planet", planet->Name());
 		
 		for(const DataNode &change : changes)
 			out.Write(change);
@@ -104,4 +109,6 @@ void GameEvent::Apply(PlayerInfo &player)
 	
 	for(const System *system : systemsToUnvisit)
 		player.Unvisit(system);
+	for(const Planet *planet : planetsToUnvisit)
+		player.Unvisit(planet);
 }
