@@ -30,7 +30,6 @@ namespace {
 
 // Default constructor.
 Government::Government()
-	: name("Uninhabited")
 {
 	// Default penalties:
 	penaltyFor[ShipEvent::ASSIST] = -0.1;
@@ -102,8 +101,12 @@ void Government::Load(const DataNode &node)
 			deathSentence = GameData::Conversations().Get(child.Token(1));
 		else if(child.Token(0) == "friendly hail" && child.Size() >= 2)
 			friendlyHail = GameData::Phrases().Get(child.Token(1));
+		else if(child.Token(0) == "friendly disabled hail" && child.Size() >= 2)
+			friendlyDisabledHail = GameData::Phrases().Get(child.Token(1));
 		else if(child.Token(0) == "hostile hail" && child.Size() >= 2)
 			hostileHail = GameData::Phrases().Get(child.Token(1));
+		else if(child.Token(0) == "hostile disabled hail" && child.Size() >= 2)
+			hostileDisabledHail = GameData::Phrases().Get(child.Token(1));
 		else if(child.Token(0) == "language" && child.Size() >= 2)
 			language = child.Token(1);
 		else if(child.Token(0) == "raid" && child.Size() >= 2)
@@ -111,6 +114,12 @@ void Government::Load(const DataNode &node)
 		else
 			child.PrintTrace("Skipping unrecognized attribute:");
 	}
+	
+	// Default to the standard disabled hail messages.
+	if(!friendlyDisabledHail)
+		friendlyDisabledHail = GameData::Phrases().Get("friendly disabled");
+	if(!hostileDisabledHail)
+		hostileDisabledHail = GameData::Phrases().Get("hostile disabled");
 }
 
 
@@ -198,10 +207,17 @@ const Conversation *Government::DeathSentence() const
 
 
 
-// Get a random hail message (depending on whether this is an enemy government).
-string Government::GetHail() const
+// Get a hail message (which depends on whether this is an enemy government
+// and if the ship is disabled).
+string Government::GetHail(bool isDisabled) const
 {
-	const Phrase *phrase = IsEnemy() ? hostileHail : friendlyHail;
+	const Phrase *phrase = nullptr;
+	
+	if(IsEnemy())
+		phrase = isDisabled ? hostileDisabledHail : hostileHail;
+	else
+		phrase = isDisabled ? friendlyDisabledHail : friendlyHail;
+		
 	return phrase ? phrase->Get() : "";
 }
 
