@@ -18,10 +18,12 @@ using namespace std;
 
 
 
-void Information::SetSprite(const string &name, const Sprite *sprite, const Point &unit)
+void Information::SetSprite(const string &name, const Sprite *sprite, const Point &unit, int frame)
 {
 	sprites[name] = sprite;
 	spriteUnits[name] = unit;
+	if(frame)
+		spriteFrames[name] = frame;
 }
 
 
@@ -36,7 +38,7 @@ const Sprite *Information::GetSprite(const string &name) const
 
 
 
-const Point &Information::GetSpriteUnit(const std::string &name) const
+const Point &Information::GetSpriteUnit(const string &name) const
 {
 	static const Point up(0., -1.);
 	
@@ -46,6 +48,14 @@ const Point &Information::GetSpriteUnit(const std::string &name) const
 
 
 
+int Information::GetSpriteFrame(const string &name) const
+{
+	auto it = spriteFrames.find(name);
+	return (it == spriteFrames.end()) ? 0 : it->second;
+}
+
+
+
 void Information::SetString(const string &name, const string &value)
 {
 	strings[name] = value;
@@ -104,7 +114,7 @@ bool Information::HasCondition(const string &condition) const
 	if(condition.front() == '!')
 		return !HasCondition(condition.substr(1));
 	
-	return (conditions.find(condition) != conditions.end());
+	return conditions.count(condition);
 }
 
 
