@@ -15,10 +15,13 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "Panel.h"
 
+#include "Point.h"
 #include "SavedGame.h"
 
+#include <ctime>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
 class PlayerInfo;
@@ -32,7 +35,20 @@ class LoadPanel : public Panel {
 public:
 	LoadPanel(PlayerInfo &player, UI &gamePanels);
 	
-	virtual void Draw() const override;
+	virtual void Draw() override;
+	
+	
+protected:
+	// Only override the ones you need; the default action is to return false.
+	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command) override;
+	virtual bool Click(int x, int y, int clicks) override;
+	virtual bool Hover(int x, int y) override;
+	virtual bool Drag(double dx, double dy) override;
+	virtual bool Scroll(double dx, double dy) override;
+	
+	
+private:
+	void UpdateLists();
 	
 	// New player "conversation" callback.
 	void OnCallback(int value);
@@ -40,19 +56,7 @@ public:
 	void SnapshotCallback(const std::string &name);
 	// Load snapshot callback.
 	void LoadCallback();
-	
-	
-protected:
-	// Only override the ones you need; the default action is to return false.
-	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command) override;
-	virtual bool Click(int x, int y) override;
-	virtual bool Hover(int x, int y) override;
-	virtual bool Drag(int dx, int dy) override;
-	virtual bool Scroll(int dx, int dy) override;
-	
-	
-private:
-	void UpdateLists();
+	// Delete callbacks.
 	void DeletePilot();
 	void DeleteSave();
 	
@@ -62,13 +66,16 @@ private:
 	SavedGame loadedInfo;
 	UI &gamePanels;
 	
-	std::map<std::string, std::vector<std::string>> files;
+	std::map<std::string, std::vector<std::pair<std::string, std::time_t>>> files;
 	std::string selectedPilot;
 	std::string selectedFile;
 	
+	Point hoverPoint;
+	int hoverCount = 0;
+	bool hasHover = false;
 	bool sideHasFocus = false;
-	int sideScroll = 0;
-	int centerScroll = 0;
+	double sideScroll = 0;
+	double centerScroll = 0;
 };
 
 
