@@ -29,10 +29,10 @@ class ShipEvent;
 // needed to move the ships around and to figure out where they should be drawn.
 class MainPanel : public Panel {
 public:
-	MainPanel(PlayerInfo &player);
+	explicit MainPanel(PlayerInfo &player);
 	
 	virtual void Step() override;
-	virtual void Draw() const override;
+	virtual void Draw() override;
 	
 	// The planet panel calls this when it closes.
 	void OnCallback();
@@ -41,12 +41,16 @@ public:
 protected:
 	// Only override the ones you need; the default action is to return false.
 	virtual bool KeyDown(SDL_Keycode key, Uint16 mod, const Command &command) override;
-	virtual bool Click(int x, int y) override;
+	virtual bool Click(int x, int y, int clicks) override;
+	virtual bool RClick(int x, int y) override;
+	virtual bool Drag(double dx, double dy) override;
+	virtual bool Release(int x, int y) override;
+	virtual bool Scroll(double dx, double dy) override;
 	
 	
 private:
 	void ShowScanDialog(const ShipEvent &event);
-	void ShowHailPanel();
+	bool ShowHailPanel();
 	
 	
 private:
@@ -56,9 +60,16 @@ private:
 	
 	Command show;
 	
-	mutable double load;
-	mutable double loadSum;
-	mutable int loadCount;
+	double load;
+	double loadSum;
+	int loadCount;
+	
+	Point dragSource;
+	Point dragPoint;
+	bool isDragging = false;
+	bool hasShift = false;
+	bool canClick = false;
+	bool canDrag = false;
 };
 
 
