@@ -13,18 +13,27 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "MapPanel.h"
 
 #include "Angle.h"
-#include "DotShader.h"
+#include "Dialog.h"
+#include "FogShader.h"
 #include "Font.h"
 #include "FontSet.h"
 #include "Galaxy.h"
 #include "GameData.h"
 #include "Government.h"
+#include "Information.h"
+#include "Interface.h"
 #include "LineShader.h"
+#include "MapDetailPanel.h"
+#include "MapOutfitterPanel.h"
+#include "MapShipyardPanel.h"
 #include "Mission.h"
+#include "MissionPanel.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
 #include "PointerShader.h"
 #include "Politics.h"
+#include "Preferences.h"
+#include "RingShader.h"
 #include "Screen.h"
 #include "Ship.h"
 #include "SpriteShader.h"
@@ -35,18 +44,11 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include <algorithm>
 #include <cctype>
+#include <cmath>
+#include <limits>
 
 using namespace std;
 
-namespace {
-	bool Contains(const string &str, const string &sub)
-	{
-		auto it = search(str.begin(), str.end(), sub.begin(), sub.end(),
-			[](char a, char b) { return toupper(a) == toupper(b); });
-		return (it != str.end());
-	}
-}
-
 const double MapPanel::OUTER = 6.;
 const double MapPanel::INNER = 3.5;
 
@@ -61,43 +63,34 @@ MapPanel::MapPanel(PlayerInfo &player, int commodity, const System *special)
 {
 	SetIsFullScreen(true);
 	SetInterruptible(false);
+	// Recalculate the fog each time the map is opened, just in case the player
+	// bought a map since the last time they viewed the map.
+	FogShader::Redraw();
 	
 	if(selectedSystem)
-		center = Point(0., 0.) - Zoom() * (selectedSystem->Position());
+		center = Point(0., 0.) - selectedSystem->Position();
 }
 
 
 
-void MapPanel::SetCommodity(int index)
-{
-	commodity = index;
-}
-
-
-
-void MapPanel::Step()
-{
-	if(tradeCommodity && commodity > 0)
-		*tradeCommodity = commodity;
-}
-
-
-
-void MapPanel::Draw() const
+void MapPanel::Draw()
 {
 	glClear(GL_COLOR_BUFFER_BIT);
 	
 	for(const auto &it : GameData::Galaxies())
 		SpriteShader::Draw(it.second.GetSprite(), Zoom() * (center + it.second.Position()), Zoom());
 	
+	if(Preferences::Has("Hide unexplored map regions"))
+		FogShader::Draw(center, Zoom(), player);
+	
 	DrawTravelPlan();
 	
 	// Draw the "visible range" circle around your current location.
 	Color dimColor(.1, 0.);
-	DotShader::Draw(Zoom() * (playerSystem ? playerSystem->Position() + center : center),
-		100.5 * Zoom(), 99.5 * Zoom(), dimColor);
+	RingShader::Draw(Zoom() * (playerSystem ? playerSystem->Position() + center : center),
+		(System::NEIGHBOR_DISTANCE + .5) * Zoom(), (System::NEIGHBOR_DISTANCE - .5) * Zoom(), dimColor);
 	Color brightColor(.4, 0.);
-	DotShader::Draw(Zoom() * (selectedSystem ? selectedSystem->Position() + center : center),
+	RingShader::Draw(Zoom() * (selectedSystem ? selectedSystem->Position() + center : center),
 		11., 9., brightColor);
 	
 	DrawWormholes();
@@ -122,7 +115,176 @@ void MapPanel::Draw() const
 
 
 
-bool MapPanel::Click(int x, int y)
+void MapPanel::DrawButtons(const string &condition)
+{
+	// Remember which buttons we're showing.
+	buttonCondition = condition;
+	
+	// Draw the buttons to switch to other map modes.
+	Information info;
+	info.SetCondition(condition);
+	if(player.MapZoom() == 2)
+		info.SetCondition("max zoom");
+	if(player.MapZoom() == -2)
+		info.SetCondition("min zoom");
+	const Interface *interface = GameData::Interfaces().Get("map buttons");
+	interface->Draw(info, this);
+}
+
+
+
+void MapPanel::DrawMiniMap(const PlayerInfo &player, double alpha, const System *const jump[2], int step)
+{
+	const Font &font = FontSet::Get(14);
+	Color lineColor(alpha, 0.);
+	Point center = .5 * (jump[0]->Position() + jump[1]->Position());
+	Point drawPos(0., Screen::Top() + 100.);
+	set<const System *> seen;
+	bool isLink = false;
+
+	const Set<Color> &colors = GameData::Colors();
+	Color currentColor = colors.Get("active mission")->Additive(alpha * 2.);
+	Color blockedColor = colors.Get("blocked mission")->Additive(alpha * 2.);
+	Color waypointColor = colors.Get("waypoint")->Additive(alpha * 2.);
+	
+	for(int i = 0; i < 2; ++i)
+	{
+		const System *system = jump[i];
+		const Government *gov = system->GetGovernment();
+		bool isKnown = player.KnowsName(system);
+		Point from = system->Position() - center + drawPos;
+		string name = isKnown ? system->Name() : "Unexplored System";
+		font.Draw(name, from + Point(6., -.5 * font.Height()), lineColor);
+		
+		Color color = Color(.5 * alpha, 0.);
+		if(player.HasVisited(system) && system->IsInhabited(player.Flagship()) && gov)
+			color = Color(
+				alpha * gov->GetColor().Get()[0],
+				alpha * gov->GetColor().Get()[1],
+				alpha * gov->GetColor().Get()[2], 0.);
+		RingShader::Draw(from, 6., 3.5, color);
+		
+		for(const System *link : system->Links())
+		{
+			if(!player.HasVisited(system) && !player.HasVisited(link))
+				continue;
+			
+			Point to = link->Position() - center + drawPos;
+			Point unit = (from - to).Unit() * 7.;
+			LineShader::Draw(from - unit, to + unit, 1.2, lineColor);
+			
+			isLink |= (link == jump[!i]);
+			if(seen.count(link) || link == jump[!i])
+				continue;
+			seen.insert(link);
+			
+			gov = link->GetGovernment();
+			Color color = Color(.5 * alpha, 0.);
+			if(player.HasVisited(link) && link->IsInhabited(player.Flagship()) && gov)
+				color = Color(
+					alpha * gov->GetColor().Get()[0],
+					alpha * gov->GetColor().Get()[1],
+					alpha * gov->GetColor().Get()[2], 0.);
+			RingShader::Draw(to, 6., 3.5, color);
+		}
+		
+		Angle angle;
+		for(const Mission &mission : player.Missions())
+		{
+			if(!mission.IsVisible())
+				continue;
+			
+			if(mission.Destination()->IsInSystem(system))
+			{
+				bool blink = false;
+				if(mission.Deadline())
+				{
+					int days = min(5, mission.Deadline() - player.GetDate()) + 1;
+					if(days > 0)
+						blink = (step % (10 * days) > 5 * days);
+				}
+				if(!blink)
+				{
+					bool isSatisfied = IsSatisfied(player, mission);
+					DrawPointer(from, angle, isSatisfied ? currentColor : blockedColor, false);
+				}
+			}
+			
+			for(const System *waypoint : mission.Waypoints())
+				if(waypoint == system)
+					DrawPointer(from, angle, waypointColor, false);
+			for(const Planet *stopover : mission.Stopovers())
+				if(stopover->IsInSystem(system))
+					DrawPointer(from, angle, waypointColor, false);
+		}
+	}
+	
+	Point from = jump[0]->Position() - center + drawPos;
+	Point to = jump[1]->Position() - center + drawPos;
+	Point unit = (to - from).Unit();
+	from += 7. * unit;
+	to -= 7. * unit;
+	Color bright(2. * alpha, 0.);
+	if(!isLink)
+	{
+		double length = (to - from).Length();
+		int segments = static_cast<int>(length / 15.);
+		for(int i = 0; i < segments; ++i)
+			LineShader::Draw(
+				from + unit * ((i * length) / segments + 2.),
+				from + unit * (((i + 1) * length) / segments - 2.),
+				1.2, bright);
+	}
+	LineShader::Draw(to, to + Angle(-30.).Rotate(unit) * -10., 1.2, bright);
+	LineShader::Draw(to, to + Angle(30.).Rotate(unit) * -10., 1.2, bright);
+}
+
+
+
+bool MapPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
+{
+	if(command.Has(Command::MAP) || key == 'd' || key == SDLK_ESCAPE
+			|| (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
+		GetUI()->Pop(this);
+	else if(key == 's' && buttonCondition != "is shipyards")
+	{
+		GetUI()->Pop(this);
+		GetUI()->Push(new MapShipyardPanel(*this));
+	}
+	else if(key == 'o' && buttonCondition != "is outfitters")
+	{
+		GetUI()->Pop(this);
+		GetUI()->Push(new MapOutfitterPanel(*this));
+	}
+	else if(key == 'i' && buttonCondition != "is missions")
+	{
+		GetUI()->Pop(this);
+		GetUI()->Push(new MissionPanel(*this));
+	}
+	else if(key == 'p' && buttonCondition != "is ports")
+	{
+		GetUI()->Pop(this);
+		GetUI()->Push(new MapDetailPanel(*this));
+	}
+	else if(key == 'f')
+	{
+		GetUI()->Push(new Dialog(
+			this, &MapPanel::Find, "Search for:"));
+		return true;
+	}
+	else if(key == '+' || key == '=')
+		player.SetMapZoom(min(2, player.MapZoom() + 1));
+	else if(key == '-')
+		player.SetMapZoom(max(-2, player.MapZoom() - 1));
+	else
+		return false;
+	
+	return true;
+}
+
+
+
+bool MapPanel::Click(int x, int y, int clicks)
 {
 	// Figure out if a system was clicked on.
 	Point click = Point(x, y) / Zoom() - center;
@@ -139,7 +301,7 @@ bool MapPanel::Click(int x, int y)
 
 
 
-bool MapPanel::Drag(int dx, int dy)
+bool MapPanel::Drag(double dx, double dy)
 {
 	center += Point(dx, dy) / Zoom();
 	return true;
@@ -147,15 +309,16 @@ bool MapPanel::Drag(int dx, int dy)
 
 
 
-bool MapPanel::Scroll(int dx, int dy)
+bool MapPanel::Scroll(double dx, double dy)
 {
 	// The mouse should be pointing to the same map position before and after zooming.
 	Point mouse = UI::GetMouse();
 	Point anchor = mouse / Zoom() - center;
-	if(dy > 0)
-		ZoomMap();
+	if(dy > 0.)
+		player.SetMapZoom(min(2, player.MapZoom() + 1));
 	else
-		UnzoomMap();
+		player.SetMapZoom(max(-2, player.MapZoom() - 1));
+	
 	// Now, Zoom() has changed (unless at one of the limits). But, we still want
 	// anchor to be the same, so:
 	center = mouse / Zoom() - anchor;
@@ -166,6 +329,9 @@ bool MapPanel::Scroll(int dx, int dy)
 
 Color MapPanel::MapColor(double value)
 {
+	if(std::isnan(value))
+		return UninhabitedColor();
+	
 	value = min(1., max(-1., value));
 	if(value < 0.)
 		return Color(
@@ -238,7 +404,7 @@ Color MapPanel::UnexploredColor()
 
 double MapPanel::SystemValue(const System *system) const
 {
-	return 0;
+	return 0.;
 }
 
 
@@ -248,96 +414,95 @@ void MapPanel::Select(const System *system)
 	if(!system)
 		return;
 	selectedSystem = system;
+	vector<const System *> &plan = player.TravelPlan();
+	if(!player.Flagship() || (!plan.empty() && system == plan.front()))
+		return;
 	
-	bool shift = (SDL_GetModState() & KMOD_SHIFT) && player.HasTravelPlan();
-	if(system == playerSystem && !shift)
-		player.ClearTravel();
-	else if((distance.Distance(system) > 0 || shift) && player.Flagship())
+	bool isJumping = player.Flagship()->IsEnteringHyperspace();
+	const System *source = isJumping ? player.Flagship()->GetTargetSystem() : player.GetSystem();
+	
+	bool shift = (SDL_GetModState() & KMOD_SHIFT) && !plan.empty();
+	if(system == source && !shift)
 	{
-		if(shift)
+		plan.clear();
+		if(!isJumping)
+			player.Flagship()->SetTargetSystem(nullptr);
+		else
+			plan.push_back(source);
+	}
+	else if(shift)
+	{
+		DistanceMap localDistance(player, plan.front());
+		if(localDistance.Days(system) <= 0)
+			return;
+		
+		auto it = plan.begin();
+		while(system != *it)
 		{
-			vector<const System *> oldPath = player.TravelPlan();
-			DistanceMap localDistance(player, oldPath.front());
-			if(localDistance.Distance(system) <= 0)
-				return;
-			player.ClearTravel();
-			
-			while(system != oldPath.front())
-			{
-				player.AddTravel(system);
-				system = localDistance.Route(system);
-			}
-			for(const System *it : oldPath)
-				player.AddTravel(it);
+			it = ++plan.insert(it, system);
+			system = localDistance.Route(system);
 		}
-		else if(playerSystem)
+	}
+	else if(distance.Days(system) > 0)
+	{
+		plan.clear();
+		if(!isJumping)
+			player.Flagship()->SetTargetSystem(nullptr);
+		
+		while(system != source)
 		{
-			player.ClearTravel();
-			while(system != playerSystem)
-			{
-				player.AddTravel(system);
-				system = distance.Route(system);
-			}
+			plan.push_back(system);
+			system = distance.Route(system);
 		}
+		if(isJumping)
+			plan.push_back(source);
 	}
 }
 
 
 
-const Planet *MapPanel::Find(const string &name)
+void MapPanel::Find(const string &name)
 {
+	int bestIndex = 9999;
 	for(const auto &it : GameData::Systems())
-		if(player.HasVisited(&it.second) && Contains(it.first, name))
+		if(player.HasVisited(&it.second))
 		{
-			selectedSystem = &it.second;
-			center = Zoom() * (Point() - selectedSystem->Position());
-			return nullptr;
+			int index = Search(it.first, name);
+			if(index >= 0 && index < bestIndex)
+			{
+				bestIndex = index;
+				selectedSystem = &it.second;
+				center = Zoom() * (Point() - selectedSystem->Position());
+				if(!index)
+				{
+					selectedPlanet = nullptr;
+					return;
+				}
+			}
 		}
 	for(const auto &it : GameData::Planets())
-		if(player.HasVisited(it.second.GetSystem()) && Contains(it.first, name))
+		if(player.HasVisited(it.second.GetSystem()))
 		{
-			selectedSystem = it.second.GetSystem();
-			center = Zoom() * (Point() - selectedSystem->Position());
-			return &it.second;
+			int index = Search(it.first, name);
+			if(index >= 0 && index < bestIndex)
+			{
+				bestIndex = index;
+				selectedSystem = it.second.GetSystem();
+				center = Zoom() * (Point() - selectedSystem->Position());
+				if(!index)
+				{
+					selectedPlanet = &it.second;
+					return;
+				}
+			}
 		}
-	return nullptr;
-}
-
-
-
-void MapPanel::ZoomMap()
-{
-	if(zoom < maxZoom)
-		zoom++;
-}
-
-
-
-void MapPanel::UnzoomMap()
-{
-	if(zoom > -maxZoom)
-		zoom--;
 }
 
 
 
 double MapPanel::Zoom() const
 {
-	return pow(1.5, zoom);
-}
-
-
-
-bool MapPanel::ZoomIsMax() const
-{
-	return (zoom == maxZoom);
-}
-
-
-
-bool MapPanel::ZoomIsMin() const
-{
-	return (zoom == -maxZoom);
+	return pow(1.5, player.MapZoom());
 }
 
 
@@ -345,6 +510,13 @@ bool MapPanel::ZoomIsMin() const
 // Check whether the NPC and waypoint conditions of the given mission have
 // been satisfied.
 bool MapPanel::IsSatisfied(const Mission &mission) const
+{
+	return IsSatisfied(player, mission);
+}
+
+
+
+bool MapPanel::IsSatisfied(const PlayerInfo &player, const Mission &mission)
 {
 	for(const NPC &npc : mission.NPCs())
 		if(!npc.HasSucceeded(player.GetSystem()))
@@ -355,99 +527,92 @@ bool MapPanel::IsSatisfied(const Mission &mission) const
 
 
 
-void MapPanel::DrawTravelPlan() const
+int MapPanel::Search(const string &str, const string &sub)
 {
+	auto it = search(str.begin(), str.end(), sub.begin(), sub.end(),
+		[](char a, char b) { return toupper(a) == toupper(b); });
+	return (it == str.end() ? -1 : it - str.begin());
+}
+
+
+
+void MapPanel::DrawTravelPlan()
+{
+	if(!playerSystem)
+		return;
+	
 	Color defaultColor(.5, .4, 0., 0.);
 	Color outOfFlagshipFuelRangeColor(.55, .1, .0, 0.);
 	Color withinFleetFuelRangeColor(.2, .5, .0, 0.);
 	Color wormholeColor(0.5, 0.2, 0.9, 1.);
 	
-	Ship *ship = player.Flagship();
-	bool hasHyper = ship ? ship->Attributes().Get("hyperdrive") : false;
-	bool hasJump = ship ? ship->Attributes().Get("jump drive") : false;
-	
-	// Find out how much fuel your ship and your escorts use per jump.
-	double flagshipCapacity = 0.;
-	if(ship)
-		flagshipCapacity = ship->Attributes().Get("fuel capacity") * ship->Fuel();
-	double flagshipJumpFuel = 0.;
-	if(ship)
-		flagshipJumpFuel = hasHyper ? ship->Attributes().Get("scram drive") ? 150. : 100. : 200.;
-	double escortCapacity = 0.;
-	double escortJumpFuel = 1.;
-	bool escortHasJump = false;
-	// Skip your flagship, parked ships, and fighters.
-	for(const shared_ptr<Ship> &it : player.Ships())
-		if(it.get() != ship && !it->IsParked() && !it->CanBeCarried())
-		{
-			double capacity = it->Attributes().Get("fuel capacity") * it->Fuel();
-			double jumpFuel = it->Attributes().Get("hyperdrive") ?
-				it->Attributes().Get("scram drive") ? 150. : 100. : 200.;
-			if(escortJumpFuel < 100. || capacity / jumpFuel < escortCapacity / escortJumpFuel)
-			{
-				escortCapacity = capacity;
-				escortJumpFuel = jumpFuel;
-				escortHasJump = it->Attributes().Get("jump drive");
-			}
-		}
-	
-	// Draw your current travel plan.
-	if(!playerSystem)
+	// At each point in the path, we'll keep track of how many ships in the
+	// fleet are able to make it this far.
+	Ship *flagship = player.Flagship();
+	if(!flagship)
 		return;
+	bool stranded = false;
+	bool hasEscort = false;
+	map<const Ship *, double> fuel;
+	for(const shared_ptr<Ship> &it : player.Ships())
+		if(!it->IsParked() && !it->CanBeCarried() && it->GetSystem() == flagship->GetSystem())
+		{
+			if(it->IsDisabled())
+			{
+				stranded = true;
+				continue;
+			}
+			
+			fuel[it.get()] = it->Fuel() * it->Attributes().Get("fuel capacity");
+			hasEscort |= (it.get() != flagship);
+		}
+	stranded |= !hasEscort;
+	
 	const System *previous = playerSystem;
 	for(int i = player.TravelPlan().size() - 1; i >= 0; --i)
 	{
 		const System *next = player.TravelPlan()[i];
-		
-		// Figure out what kind of jump this is, and check if the player is able
-		// to make jumps of that kind.
-		bool isHyper = 
-			(find(previous->Links().begin(), previous->Links().end(), next)
-				!= previous->Links().end());
-		bool isJump = isHyper ||
-			(find(previous->Neighbors().begin(), previous->Neighbors().end(), next)
-				!= previous->Neighbors().end());
+		bool isHyper = previous->Links().count(next);
+		bool isJump = !isHyper && previous->Neighbors().count(next);
 		bool isWormhole = false;
-		if(!((isHyper && hasHyper) || (isJump && hasJump)))
-		{
-			for(const StellarObject &object : previous->Objects())
-				isWormhole |= (object.GetPlanet() && object.GetPlanet()->WormholeDestination(previous) == next);
-			if(!isWormhole)
-				break;
-		}
+		for(const StellarObject &object : previous->Objects())
+			isWormhole |= (object.GetPlanet() && player.HasVisited(object.GetPlanet())
+				&& player.HasVisited(previous) && player.HasVisited(next)
+				&& object.GetPlanet()->WormholeDestination(previous) == next);
+		
+		if(!isHyper && !isJump && !isWormhole)
+			break;
+		
+		// Wormholes cost nothing to grow through. If this is not a wormhole,
+		// check how much fuel every ship will expend to go through it.
+		if(!isWormhole)
+			for(auto &it : fuel)
+				if(it.second >= 0.)
+				{
+					double cost = isJump ? it.first->JumpDriveFuel() : it.first->HyperdriveFuel();
+					if(!cost || cost > it.second)
+					{
+						it.second = -1.;
+						stranded = true;
+					}
+					else
+						it.second -= cost;
+				}
+		
+		// Color the path green if all ships can make it. Color it yellow if
+		// the flagship can make it, and red if the flagship cannot.
+		Color drawColor = outOfFlagshipFuelRangeColor;
+		if(isWormhole)
+			drawColor = wormholeColor;
+		else if(!stranded)
+			drawColor = withinFleetFuelRangeColor;
+		else if(fuel[flagship] >= 0.)
+			drawColor = defaultColor;
 		
 		Point from = Zoom() * (next->Position() + center);
 		Point to = Zoom() * (previous->Position() + center);
 		Point unit = (from - to).Unit() * 7.;
-		from -= unit;
-		to += unit;
-		
-		if(isWormhole)
-		{
-			// Wormholes cost no fuel to travel through.
-		}
-		else if(!isHyper)
-		{
-			if(!escortHasJump)
-				escortCapacity = 0.;
-			flagshipCapacity -= 200.;
-			escortCapacity -= 200.;
-		}
-		else
-		{
-			flagshipCapacity -= flagshipJumpFuel;
-			escortCapacity -= escortJumpFuel;
-		}
-		
-		Color drawColor = outOfFlagshipFuelRangeColor;
-		if(isWormhole)
-			drawColor = wormholeColor;
-		else if(flagshipCapacity >= 0. && escortCapacity >= 0.)
-			drawColor = withinFleetFuelRangeColor;
-		else if(flagshipCapacity >= 0. || escortCapacity >= 0.)
-			drawColor = defaultColor;
-        
-		LineShader::Draw(from, to, 3., drawColor);
+		LineShader::Draw(from - unit, to + unit, 3., drawColor);
 		
 		previous = next;
 	}
@@ -455,7 +620,7 @@ void MapPanel::DrawTravelPlan() const
 
 
 
-void MapPanel::DrawWormholes() const
+void MapPanel::DrawWormholes()
 {
 	Color wormholeColor(0.5, 0.2, 0.9, 1.);
 	Color wormholeDimColor(0.5 / 3., 0.2 / 3., 0.9 / 3., 1.);
@@ -471,6 +636,10 @@ void MapPanel::DrawWormholes() const
 			if(object.GetPlanet() && object.GetPlanet()->IsWormhole() && player.HasVisited(object.GetPlanet()))
 			{
 				const System *next = object.GetPlanet()->WormholeDestination(previous);
+				// Only draw a wormhole if both systems have been visited.
+				if(!player.HasVisited(previous) || !player.HasVisited(next))
+					continue;
+				
 				drawn[previous] = next;
 				Point from = Zoom() * (previous->Position() + center);
 				Point to = Zoom() * (next->Position() + center);
@@ -497,7 +666,7 @@ void MapPanel::DrawWormholes() const
 
 
 
-void MapPanel::DrawLinks() const
+void MapPanel::DrawLinks()
 {
 	// Draw the links between the systems.
 	Color closeColor(.6, .6);
@@ -531,7 +700,7 @@ void MapPanel::DrawLinks() const
 
 
 
-void MapPanel::DrawSystems() const
+void MapPanel::DrawSystems()
 {
 	if(commodity == SHOW_GOVERNMENT)
 		closeGovernments.clear();
@@ -553,7 +722,7 @@ void MapPanel::DrawSystems() const
 		Color color = UninhabitedColor();
 		if(!player.HasVisited(&system))
 			color = UnexploredColor();
-		else if(system.IsInhabited())
+		else if(system.IsInhabited(player.Flagship()) || commodity == SHOW_SPECIAL)
 		{
 			if(commodity >= SHOW_SPECIAL)
 			{
@@ -561,8 +730,11 @@ void MapPanel::DrawSystems() const
 				if(commodity >= 0)
 				{
 					const Trade::Commodity &com = GameData::Commodities()[commodity];
-					value = (2. * (system.Trade(com.name) - com.low))
-						/ (com.high - com.low) - 1.;
+					double price = system.Trade(com.name);
+					if(!price)
+						value = numeric_limits<double>::quiet_NaN();
+					else
+						value = (2. * (price - com.low)) / (com.high - com.low) - 1.;
 				}
 				else if(commodity == SHOW_SHIPYARD)
 				{
@@ -585,7 +757,7 @@ void MapPanel::DrawSystems() const
 					bool all = true;
 					bool some = false;
 					for(const StellarObject &object : system.Objects())
-						if(object.GetPlanet())
+						if(object.GetPlanet() && !object.GetPlanet()->IsWormhole())
 						{
 							bool visited = player.HasVisited(object.GetPlanet());
 							all &= visited;
@@ -617,29 +789,46 @@ void MapPanel::DrawSystems() const
 			{
 				double reputation = system.GetGovernment()->Reputation();
 				
+				// A system should show up as dominated if it contains at least
+				// one inhabited planet and all inhabited planets have been
+				// dominated. It should show up as restricted if you cannot land
+				// on any of the planets that have spaceports.
 				bool hasDominated = true;
+				bool isInhabited = false;
 				bool canLand = false;
+				bool hasSpaceport = false;
 				for(const StellarObject &object : system.Objects())
-					if(object.GetPlanet() && object.GetPlanet()->HasSpaceport())
+					if(object.GetPlanet())
 					{
-						canLand |= object.GetPlanet()->CanLand();
-						hasDominated &= (!object.GetPlanet()->IsInhabited()
-							|| GameData::GetPolitics().HasDominated(object.GetPlanet()));
+						const Planet *planet = object.GetPlanet();
+						hasSpaceport |= !planet->IsWormhole() && planet->HasSpaceport();
+						if(planet->IsWormhole() || !planet->IsAccessible(player.Flagship()))
+							continue;
+						canLand |= planet->CanLand() && planet->HasSpaceport();
+						isInhabited |= planet->IsInhabited();
+						hasDominated &= (!planet->IsInhabited()
+							|| GameData::GetPolitics().HasDominated(planet));
 					}
-				color = ReputationColor(reputation, canLand, canLand && hasDominated);
+				hasDominated &= (isInhabited && canLand);
+				// Some systems may count as "inhabited" but not contain any
+				// planets with spaceports. Color those as if they're
+				// uninhabited to make it clear that no fuel is available there.
+				if(hasSpaceport || hasDominated)
+					color = ReputationColor(reputation, canLand, hasDominated);
 			}
 		}
 		
-		DotShader::Draw(pos, OUTER, INNER, color);
+		RingShader::Draw(pos, OUTER, INNER, color);
 	}
 }
 
 
 
-void MapPanel::DrawNames() const
+void MapPanel::DrawNames()
 {
 	// Don't draw if too small.
-	if (Zoom() <= 0.5) return;
+	if(Zoom() <= 0.5)
+		return;
 	
 	// Draw names for all systems you have visited.
 	const Font &font = FontSet::Get((Zoom() > 2.0) ? 18 : 14);
@@ -659,7 +848,7 @@ void MapPanel::DrawNames() const
 
 
 
-void MapPanel::DrawMissions() const
+void MapPanel::DrawMissions()
 {
 	// Draw a pointer for each active or available mission.
 	map<const System *, Angle> angle;
@@ -691,8 +880,8 @@ void MapPanel::DrawMissions() const
 			if(days > 0)
 				blink = (step % (10 * days) > 5 * days);
 		}
-		DrawPointer(system, angle[system],
-			blink ? black : IsSatisfied(mission) ? currentColor : blockedColor);
+		bool isSatisfied = IsSatisfied(player, mission);
+		DrawPointer(system, angle[system], blink ? black : isSatisfied ? currentColor : blockedColor, isSatisfied);
 		
 		for(const System *waypoint : mission.Waypoints())
 			DrawPointer(waypoint, angle[waypoint], waypointColor);
@@ -711,12 +900,19 @@ void MapPanel::DrawMissions() const
 
 
 
-void MapPanel::DrawPointer(const System *system, Angle &angle, const Color &color) const
+void MapPanel::DrawPointer(const System *system, Angle &angle, const Color &color, bool bigger)
+{
+	DrawPointer(Zoom() * (system->Position() + center), angle, color, true, bigger);
+}
+
+
+
+void MapPanel::DrawPointer(Point position, Angle &angle, const Color &color, bool drawBack, bool bigger)
 {
 	static const Color black(0., 1.);
 	
 	angle += Angle(30.);
-	Point pos = Zoom() * (system->Position() + center);
-	PointerShader::Draw(pos, angle.Unit(), 14., 19., -4., black);
-	PointerShader::Draw(pos, angle.Unit(), 8., 15., -6., color);
+	if(drawBack)
+		PointerShader::Draw(position, angle.Unit(), 14. + bigger, 19. + 2 * bigger, -4., black);
+	PointerShader::Draw(position, angle.Unit(), 8. + bigger, 15. + 2 * bigger, -6., color);
 }
