@@ -50,17 +50,7 @@ MenuPanel::MenuPanel(PlayerInfo &player, UI &gamePanels)
 {
 	SetIsFullScreen(true);
 	
-	string data = Files::Read(Files::Resources() + "credits.txt");
-	size_t pos = 0;
-	while(pos < data.size())
-	{
-		size_t end = data.find('\n', pos);
-		if(end == string::npos)
-			end = data.size();
-		
-		credits.push_back(data.substr(pos, end - pos));
-		pos = end + 1;
-	}
+	credits = Format::Split(Files::Read(Files::Resources() + "credits.txt"), "\n");
 }
 
 
@@ -77,14 +67,20 @@ void MenuPanel::Step()
 	if(progress == 60 && !isReady)
 	{
 		if(gamePanels.IsEmpty())
+		{
 			gamePanels.Push(new MainPanel(player));
+			// It takes one step to figure out the planet panel should be created, and
+			// another step to actually place it. So, take two steps to avoid a flicker.
+			gamePanels.StepAll();
+			gamePanels.StepAll();
+		}
 		isReady = true;
 	}
 }
 
 
 
-void MenuPanel::Draw() const
+void MenuPanel::Draw()
 {
 	glClear(GL_COLOR_BUFFER_BIT);
 	GameData::Background().Draw(Point(), Point());
@@ -97,7 +93,7 @@ void MenuPanel::Draw() const
 		if(player.Flagship())
 		{
 			const Ship &flagship = *player.Flagship();
-			info.SetSprite("ship sprite", flagship.GetSprite().GetSprite());
+			info.SetSprite("ship sprite", flagship.GetSprite());
 			info.SetString("ship", flagship.Name());
 		}
 		if(player.GetSystem())
@@ -119,8 +115,9 @@ void MenuPanel::Draw() const
 		info.SetString("pilot", "No Pilot Loaded");
 	}
 	
-	const Interface *menu = GameData::Interfaces().Get("main menu");
-	menu->Draw(info);
+	GameData::Interfaces().Get("menu background")->Draw(info, this);
+	GameData::Interfaces().Get("main menu")->Draw(info, this);
+	GameData::Interfaces().Get("menu player info")->Draw(info, this);
 	
 	if(progress == 60)
 		alpha -= .02f;
@@ -161,11 +158,11 @@ void MenuPanel::OnCallback(int)
 {
 	GetUI()->Pop(this);
 	gamePanels.Reset();
-	Panel *panel = new MainPanel(player);
-	gamePanels.Push(panel);
+	gamePanels.Push(new MainPanel(player));
 	// Tell the main panel to re-draw itself (and pop up the planet panel).
-	panel->Step();
+	gamePanels.StepAll();
 	gamePanels.Push(new ShipyardPanel(player));
+	gamePanels.StepAll();
 }
 
 
@@ -181,10 +178,9 @@ bool MenuPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 		GetUI()->Push(new PreferencesPanel());
 	else if(key == 'l')
 		GetUI()->Push(new LoadPanel(player, gamePanels));
-	else if(key == 'n' || key == 'e')
+	else if(key == 'n' && (!player.IsLoaded() || player.IsDead()))
 	{
-		// The "New Pilot" and "Enter Ship" buttons are in the same place.
-		GameData::Revert();
+		// If no player is loaded, the "Enter Ship" button becomes "New Pilot."
 		player.New();
 		
 		ConversationPanel *panel = new ConversationPanel(
@@ -199,14 +195,3 @@ bool MenuPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 	
 	return true;
 }
-
-
-
-bool MenuPanel::Click(int x, int y)
-{
-	char key = GameData::Interfaces().Get("main menu")->OnClick(Point(x, y));
-	if(key)
-		return DoKey(key);
-	
-	return true;
-}
