@@ -19,6 +19,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include <map>
 #include <set>
 #include <string>
+#include <utility>
 
 class DataNode;
 class DataWriter;
@@ -62,15 +63,18 @@ private:
 	std::string trigger;
 	std::string system;
 	
+	std::string logText;
+	std::map<std::string, std::map<std::string, std::string>> specialLogText;
+	
 	std::string dialogText;
 	
 	const Conversation *stockConversation = nullptr;
 	Conversation conversation;
 	
-	std::map<std::string, int> events;
+	std::map<std::string, std::pair<int, int>> events;
 	std::map<const Outfit *, int> gifts;
-	int payment = 0;
-	int paymentMultiplier = 0;
+	int64_t payment = 0;
+	int64_t paymentMultiplier = 0;
 	
 	// When this action is performed, the missions with these names fail.
 	std::set<std::string> fail;
