@@ -16,7 +16,6 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "Command.h"
 #include "Dialog.h"
-#include "DotShader.h"
 #include "FillShader.h"
 #include "Font.h"
 #include "FontSet.h"
@@ -24,14 +23,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Information.h"
 #include "Interface.h"
 #include "LineShader.h"
-#include "MapDetailPanel.h"
-#include "MapOutfitterPanel.h"
-#include "MapShipyardPanel.h"
 #include "Mission.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
 #include "PointerShader.h"
 #include "Preferences.h"
+#include "RingShader.h"
 #include "Screen.h"
 #include "Ship.h"
 #include "Sprite.h"
@@ -60,13 +57,29 @@ MissionPanel::MissionPanel(PlayerInfo &player)
 	while(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 		++acceptedIt;
 	
-	// Center the system slightly above the center of the screen because the
-	// lower panel is taking up more space than the upper one.
-	center = Point(0., -80.) - selectedSystem->Position();
-	
 	wrap.SetWrapWidth(380);
 	wrap.SetFont(FontSet::Get(14));
 	wrap.SetAlignment(WrappedText::JUSTIFIED);
+
+	// Select the first available or accepted mission in the currently selected
+	// system, or along the travel plan.
+	if(!FindMissionForSystem(selectedSystem) && player.HasTravelPlan())
+	{
+		const auto &tp = player.TravelPlan();
+		for(auto it = tp.crbegin(); it != tp.crend(); ++it)
+			if(FindMissionForSystem(*it))
+				break;
+	}
+
+	// Auto select the destination system for the current mission.
+	if(availableIt != available.end())
+		selectedSystem = availableIt->Destination()->GetSystem();
+	else if(acceptedIt != accepted.end())
+		selectedSystem = acceptedIt->Destination()->GetSystem();
+
+	// Center the system slightly above the center of the screen because the
+	// lower panel is taking up more space than the upper one.
+	center = Point(0., -80.) - selectedSystem->Position();
 }
 
 
@@ -79,9 +92,8 @@ MissionPanel::MissionPanel(const MapPanel &panel)
 	acceptedIt(player.AvailableJobs().empty() ? accepted.begin() : accepted.end()),
 	availableScroll(0), acceptedScroll(0), dragSide(0)
 {
-	// Don't use the "special" coloring in this view.
-	if(commodity == SHOW_SPECIAL)
-		commodity = SHOW_REPUTATION;
+	// In this view, always color systems based on player reputation.
+	commodity = SHOW_REPUTATION;
 	
 	while(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 		++acceptedIt;
@@ -89,37 +101,36 @@ MissionPanel::MissionPanel(const MapPanel &panel)
 	wrap.SetWrapWidth(380);
 	wrap.SetFont(FontSet::Get(14));
 	wrap.SetAlignment(WrappedText::JUSTIFIED);
+
+	// Select the first available or accepted mission in the currently selected
+	// system, or along the travel plan.
+	if(!FindMissionForSystem(selectedSystem)
+		&& !(player.GetSystem() != selectedSystem && FindMissionForSystem(player.GetSystem()))
+		&& player.HasTravelPlan())
+	{
+		const auto &tp = player.TravelPlan();
+		for(auto it = tp.crbegin(); it != tp.crend(); ++it)
+			if(FindMissionForSystem(*it))
+				break;
+	}
 }
 
 
 
 void MissionPanel::Step()
 {
-	if(!Preferences::Has("help: jobs"))
-	{
-		Preferences::Set("help: jobs");
-		GetUI()->Push(new Dialog(
-			"Taking on jobs is a safe way to earn money. "
-			"Special missions are offered in the Space Port; "
-			"more mundane jobs are posted on the Job Board. "
-			"Most special missions are only offered once: "
-			"if you turn one down, it will not be offered to you again.\n"
-			"\tThe payment for a job depends on how far you must travel and how much you are carrying. "
-			"Jobs that have a time limit pay extra, but you are paid nothing if you miss the deadline.\n"
-			"\tAs you gain a combat reputation, new jobs will become available, "
-			"including escorting convoys and bounty hunting."));
-	}
+	DoHelp("jobs");
 }
 
 
 
-void MissionPanel::Draw() const
+void MissionPanel::Draw()
 {
 	MapPanel::Draw();
 	
 	Color routeColor(.2, .1, 0., 0.);
 	const System *system = selectedSystem;
-	while(distance.Distance(system) > 0)
+	while(distance.Days(system) > 0)
 	{
 		const System *next = distance.Route(system);
 		
@@ -160,15 +171,7 @@ void MissionPanel::Draw() const
 	if(acceptedIt != accepted.end() && acceptedIt->Destination())
 		DrawMissionSystem(*acceptedIt, IsSatisfied(*acceptedIt) ? currentColor : blockedColor);
 	
-	// Draw the buttons to switch to other map modes.
-	Information info;
-	info.SetCondition("is missions");
-	if(ZoomIsMax())
-		info.SetCondition("max zoom");
-	if(ZoomIsMin())
-		info.SetCondition("min zoom");
-	const Interface *interface = GameData::Interfaces().Get("map buttons");
-	interface->Draw(info);
+	DrawButtons("is missions");
 }
 
 
@@ -176,43 +179,24 @@ void MissionPanel::Draw() const
 // Only override the ones you need; the default action is to return false.
 bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 {
-	if(key == 'd' || key == SDLK_ESCAPE || (key == 'w' && (mod & (KMOD_CTRL | KMOD_GUI))))
+	if(key == 'a' && CanAccept())
 	{
-		GetUI()->Pop(this);
+		Accept();
 		return true;
 	}
-	else if(key == 'a')
+	else if(key == 'A' || (key == 'a' && (mod & KMOD_SHIFT)))
 	{
-		if(CanAccept())
-			Accept();
-		else if(acceptedIt != accepted.end())
-		{
+		if(acceptedIt != accepted.end() && acceptedIt->IsVisible())
 			GetUI()->Push(new Dialog(this, &MissionPanel::AbortMission,
 				"Abort mission \"" + acceptedIt->Name() + "\"?"));
-		}
 		return true;
 	}
-	else if(key == 'p' || key == SDLK_PAGEUP || key == SDLK_PAGEDOWN)
-	{
-		GetUI()->Pop(this);
-		GetUI()->Push(new MapDetailPanel(*this));
-	}
-	else if(key == 'o')
-	{
-		GetUI()->Pop(this);
-		GetUI()->Push(new MapOutfitterPanel(*this));
-	}
-	else if(key == 's')
-	{
-		GetUI()->Pop(this);
-		GetUI()->Push(new MapShipyardPanel(*this));
-	}
 	else if(key == SDLK_LEFT && availableIt == available.end())
 	{
 		acceptedIt = accepted.end();
 		availableIt = available.begin();
 	}
-	else if(key == SDLK_RIGHT && acceptedIt == accepted.end())
+	else if(key == SDLK_RIGHT && acceptedIt == accepted.end() && AcceptedVisible())
 	{
 		availableIt = available.end();
 		acceptedIt = accepted.begin();
@@ -221,6 +205,7 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 	}
 	else if(key == SDLK_UP)
 	{
+		SelectAnyMission();
 		if(availableIt != available.end())
 		{
 			if(availableIt == available.begin())
@@ -236,7 +221,7 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			} while(!acceptedIt->IsVisible());
 		}
 	}
-	else if(key == SDLK_DOWN)
+	else if(key == SDLK_DOWN && !SelectAnyMission())
 	{
 		if(availableIt != available.end())
 		{
@@ -253,25 +238,8 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 			} while(!acceptedIt->IsVisible());
 		}
 	}
-	else if(command.Has(Command::MAP))
-	{
-		GetUI()->Pop(this);
-		GetUI()->Push(new MapDetailPanel(*this));
-		return true;
-	}
-	else if(key == 'f')
-	{
-		GetUI()->Push(new Dialog(
-			this, &MissionPanel::DoFind, "Search for:"));
-		return true;
-	}
-	else if(key == '+' || key == '=')
-		ZoomMap();
-	else if(key == '-')
-		UnzoomMap();
-
 	else
-		return false;
+		return MapPanel::KeyDown(key, mod, command);
 	
 	if(availableIt != available.end())
 		selectedSystem = availableIt->Destination()->GetSystem();
@@ -285,32 +253,16 @@ bool MissionPanel::KeyDown(SDL_Keycode key, Uint16 mod, const Command &command)
 
 
 
-bool MissionPanel::Click(int x, int y)
+bool MissionPanel::Click(int x, int y, int clicks)
 {
 	dragSide = 0;
 	
-	// Handle clicks on the interface buttons.
-	{
-		const Interface *interface = GameData::Interfaces().Get("mission");
-		char key = interface->OnClick(Point(x, y));
-		if(key)
-			return DoKey(key);
-	}
-	{
-		const Interface *interface = GameData::Interfaces().Get("map buttons");
-		char key = interface->OnClick(Point(x, y));
-		// In the mission panel, the "Done" button in the button bar should be
-		// ignored (and is not shown).
-		if(key && key != 'd')
-			return DoKey(key);
-	}
-	
 	if(x > Screen::Right() - 80 && y > Screen::Bottom() - 50)
 		return DoKey('p');
 	
 	if(x < Screen::Left() + SIDE_WIDTH)
 	{
-		unsigned index = max(0, (y + availableScroll - 36 - Screen::Top()) / 20);
+		unsigned index = max(0, (y + static_cast<int>(availableScroll) - 36 - Screen::Top()) / 20);
 		if(index < available.size())
 		{
 			availableIt = available.begin();
@@ -325,7 +277,7 @@ bool MissionPanel::Click(int x, int y)
 	}
 	else if(x >= Screen::Right() - SIDE_WIDTH)
 	{
-		int index = max(0, (y + acceptedScroll - 36 - Screen::Top()) / 20);
+		int index = max(0, (y + static_cast<int>(acceptedScroll) - 36 - Screen::Top()) / 20);
 		if(index < AcceptedVisible())
 		{
 			acceptedIt = accepted.begin();
@@ -392,11 +344,15 @@ bool MissionPanel::Click(int x, int y)
 			if(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
 				continue;
 			
-			if(availableIt != available.end() && availableIt->Destination()->GetSystem() == system)
+			if(availableIt != available.end() && availableIt->Destination()->IsInSystem(system))
 				break;
-			if(acceptedIt != accepted.end() && acceptedIt->Destination()->GetSystem() == system)
+			if(acceptedIt != accepted.end() && acceptedIt->Destination()->IsInSystem(system))
 				break;
 		}
+		// Make sure invisible missions are never selected, even if there were
+		// no other missions in this system.
+		if(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
+			acceptedIt = accepted.end();
 	}
 	
 	return true;
@@ -404,18 +360,18 @@ bool MissionPanel::Click(int x, int y)
 
 
 
-bool MissionPanel::Drag(int dx, int dy)
+bool MissionPanel::Drag(double dx, double dy)
 {
 	if(dragSide < 0)
 	{
-		availableScroll = max(0,
-			min(static_cast<int>(available.size() * 20 + 190 - Screen::Height()),
+		availableScroll = max(0.,
+			min(available.size() * 20. + 190. - Screen::Height(),
 				availableScroll - dy));
 	}
 	else if(dragSide > 0)
 	{
-		acceptedScroll = max(0,
-			min(static_cast<int>(accepted.size() * 20 + 160 - Screen::Height()),
+		acceptedScroll = max(0.,
+			min(accepted.size() * 20. + 160. - Screen::Height(),
 				acceptedScroll - dy));
 	}
 	else
@@ -429,7 +385,7 @@ bool MissionPanel::Drag(int dx, int dy)
 bool MissionPanel::Hover(int x, int y)
 {
 	dragSide = 0;
-	unsigned index = max(0, (y + availableScroll - 36 - Screen::Top()) / 20);
+	unsigned index = max(0, (y + static_cast<int>(availableScroll) - 36 - Screen::Top()) / 20);
 	if(x < Screen::Left() + SIDE_WIDTH)
 	{
 		if(index < available.size())
@@ -445,23 +401,16 @@ bool MissionPanel::Hover(int x, int y)
 
 
 
-bool MissionPanel::Scroll(int dx, int dy)
+bool MissionPanel::Scroll(double dx, double dy)
 {
 	if(dragSide)
-		return Drag(0, dy * 50);
+		return Drag(0., dy * Preferences::ScrollSpeed());
 	
 	return MapPanel::Scroll(dx, dy);
 }
 
 
 
-void MissionPanel::DoFind(const string &text)
-{
-	Find(text);
-}
-
-
-
 void MissionPanel::DrawKey() const
 {
 	const Sprite *back = SpriteSet::Get("ui/mission key");
@@ -517,7 +466,7 @@ void MissionPanel::DrawSelectedSystem() const
 		text = "Selected system: none";
 	else if(!player.KnowsName(selectedSystem))
 		text = "Selected system: unexplored system";
-	else	
+	else
 		text = "Selected system: " + selectedSystem->Name();
 	
 	int jumps = 0;
@@ -526,12 +475,8 @@ void MissionPanel::DrawSelectedSystem() const
 	if(it != plan.end())
 		jumps = plan.end() - it;
 	else if(distance.HasRoute(selectedSystem))
-	{
-		// Figure out how many jumps (not how much fuel) getting to the selected
-		// system will take.
-		for(const System *system = selectedSystem; system != player.GetSystem(); system = distance.Route(system))
-			++jumps;
-	}
+		jumps = distance.Days(selectedSystem);
+	
 	if(jumps == 1)
 		text += " (1 jump away)";
 	else if(jumps > 0)
@@ -549,11 +494,11 @@ void MissionPanel::DrawMissionSystem(const Mission &mission, const Color &color)
 	const Color &waypointColor = *GameData::Colors().Get("waypoint back");
 	
 	Point pos = Zoom() * (mission.Destination()->GetSystem()->Position() + center);
-	DotShader::Draw(pos, 22., 20.5, color);
+	RingShader::Draw(pos, 22., 20.5, color);
 	for(const System *system : mission.Waypoints())
-		DotShader::Draw(Zoom() * (system->Position() + center), 22., 20.5, waypointColor);
+		RingShader::Draw(Zoom() * (system->Position() + center), 22., 20.5, waypointColor);
 	for(const Planet *planet : mission.Stopovers())
-		DotShader::Draw(Zoom() * (planet->GetSystem()->Position() + center), 22., 20.5, waypointColor);
+		RingShader::Draw(Zoom() * (planet->GetSystem()->Position() + center), 22., 20.5, waypointColor);
 }
 
 
@@ -633,7 +578,7 @@ Point MissionPanel::DrawList(const list<Mission> &list, Point pos) const
 
 
 
-void MissionPanel::DrawMissionInfo() const
+void MissionPanel::DrawMissionInfo()
 {
 	Information info;
 	
@@ -643,26 +588,14 @@ void MissionPanel::DrawMissionInfo() const
 		info.SetCondition("can accept");
 	else if(acceptedIt != accepted.end())
 		info.SetCondition("can abort");
-	else if(available.size())
-		info.SetCondition("cannot accept");
-	else
-		info.SetCondition("cannot abort");
 	
-	int cargoFree = -player.Cargo().Used();
-	int bunksFree = -player.Cargo().Passengers();
-	for(const shared_ptr<Ship> &ship : player.Ships())
-		if(ship->GetSystem() == player.GetSystem() && !ship->IsDisabled() && !ship->IsParked())
-		{
-			cargoFree += ship->Attributes().Get("cargo space") - ship->Cargo().Used();
-			bunksFree += ship->Attributes().Get("bunks") - ship->Crew() - ship->Cargo().Passengers();
-		}
-	info.SetString("cargo free", to_string(cargoFree) + " tons");
-	info.SetString("bunks free", to_string(bunksFree) + " bunks");
+	info.SetString("cargo free", to_string(player.Cargo().Free()) + " tons");
+	info.SetString("bunks free", to_string(player.Cargo().Bunks()) + " bunks");
 	
 	info.SetString("today", player.GetDate().ToString());
 	
 	const Interface *interface = GameData::Interfaces().Get("mission");
-	interface->Draw(info);
+	interface->Draw(info, this);
 	
 	// If a mission is selected, draw its descriptive text.
 	if(availableIt != available.end())
@@ -723,7 +656,7 @@ void MissionPanel::Accept()
 		const Planet *planet = toAccept.Destination();
 		const System *system = planet->GetSystem();
 		for(auto it = available.begin(); it != available.end(); ++it)
-			if(it->Destination() && it->Destination()->GetSystem() == system)
+			if(it->Destination() && it->Destination()->IsInSystem(system))
 			{
 				availableIt = it;
 				if(it->Destination() == planet)
@@ -753,7 +686,7 @@ void MissionPanel::MakeSpaceAndAccept()
 		
 		int64_t basis = player.GetBasis(it.first, toSell);
 		player.AdjustBasis(it.first, -basis);
-		player.Cargo().Transfer(it.first, toSell);
+		player.Cargo().Remove(it.first, toSell);
 		player.Accounts().AddCredits(toSell * price);
 	}
 	
@@ -772,6 +705,8 @@ void MissionPanel::AbortMission()
 		player.RemoveMission(Mission::FAIL, toAbort, GetUI());
 		if(acceptedIt == accepted.end() && !accepted.empty())
 			--acceptedIt;
+		if(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
+			acceptedIt = accepted.end();
 	}
 }
 
@@ -784,3 +719,42 @@ int MissionPanel::AcceptedVisible() const
 		count += mission.IsVisible();
 	return count;
 }
+
+
+
+bool MissionPanel::FindMissionForSystem(const System *system)
+{
+	if(!system)
+		return false;
+
+	acceptedIt = accepted.end();
+
+	for(availableIt = available.begin(); availableIt != available.end(); ++availableIt)
+		if(availableIt->Destination()->IsInSystem(system))
+			return true;
+	for(acceptedIt = accepted.begin(); acceptedIt != accepted.end(); ++acceptedIt)
+		if(acceptedIt->IsVisible() && acceptedIt->Destination()->IsInSystem(system))
+			return true;
+
+	return false;
+}
+
+
+
+bool MissionPanel::SelectAnyMission()
+{
+	if(availableIt == available.end() && acceptedIt == accepted.end())
+	{
+		// no previous selection, reset
+		if(!available.empty())
+			availableIt = available.begin();
+		else
+		{
+			acceptedIt = accepted.begin();
+			while(acceptedIt != accepted.end() && !acceptedIt->IsVisible())
+				++acceptedIt;
+		}
+		return availableIt != available.end() || acceptedIt != accepted.end();
+	}
+	return false;
+}
