@@ -13,13 +13,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "OutfitInfoDisplay.h"
 
 #include "Color.h"
-#include "FontSet.h"
+#include "Depreciation.h"
 #include "Format.h"
-#include "GameData.h"
 #include "Outfit.h"
-#include "Table.h"
+#include "PlayerInfo.h"
 
-#include <cmath>
+#include <algorithm>
 #include <map>
 #include <set>
 #include <sstream>
@@ -27,78 +26,64 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 using namespace std;
 
 namespace {
-	static const int WIDTH = 250;
-	
-	Point Draw(Point point, const vector<string> &labels, const vector<string> &values)
-	{
-		// Get standard colors to draw with.
-		Color labelColor = *GameData::Colors().Get("medium");
-		Color valueColor = *GameData::Colors().Get("bright");
+	static const map<string, double> SCALE = {
+		{"active cooling", 60.},
+		{"afterburner energy", 60.},
+		{"afterburner fuel", 60.},
+		{"afterburner heat", 60.},
+		{"cloaking energy", 60.},
+		{"cloaking fuel", 60.},
+		{"cooling", 60.},
+		{"cooling energy", 60.},
+		{"energy consumption", 60.},
+		{"energy generation", 60.},
+		{"heat generation", 60.},
+		{"hull repair rate", 60.},
+		{"hull energy", 60.},
+		{"hull heat", 60.},
+		{"reverse thrusting energy", 60.},
+		{"reverse thrusting heat", 60.},
+		{"shield generation", 60.},
+		{"shield energy", 60.},
+		{"shield heat", 60.},
+		{"solar collection", 60.},
+		{"thrusting energy", 60.},
+		{"thrusting heat", 60.},
+		{"turn", 60.},
+		{"turning energy", 60.},
+		{"turning heat", 60.},
 		
-		Table table;
-		// Use 10-pixel margins on both sides.
-		table.AddColumn(10, Table::LEFT);
-		table.AddColumn(WIDTH - 10, Table::RIGHT);
-		table.DrawAt(point);
+		{"thrust", 60. * 60.},
+		{"reverse thrust", 60. * 60.},
+		{"afterburner thrust", 60. * 60.},
 		
-		for(unsigned i = 0; i < labels.size() && i < values.size(); ++i)
-		{
-			if(labels[i].empty())
-			{
-				table.DrawGap(10);
-				continue;
-			}
-			
-			table.Draw(labels[i], values[i].empty() ? valueColor : labelColor);
-			table.Draw(values[i], valueColor);
-		}
-		return table.GetPoint();
-	}
+		{"ion resistance", 60. * 100.},
+		{"disruption resistance", 60. * 100.},
+		{"slowing resistance", 60. * 100.}
+	};
 	
-	static const set<string> ATTRIBUTES_TO_SCALE = {
-		"afterburner energy",
-		"afterburner fuel",
-		"afterburner heat",
-		"cloaking energy",
-		"cloaking fuel",
-		"cooling",
-		"energy generation",
-		"heat generation",
-		"hull repair rate",
-		"hull energy",
-		"reverse thrusting energy",
-		"reverse thrusting heat",
-		"shield generation",
-		"shield energy",
-		"thrusting energy",
-		"thrusting heat",
-		"turn",
-		"turning energy",
-		"turning heat"
+	static const map<string, string> BOOLEAN_ATTRIBUTES = {
+		{"unplunderable", "This outfit cannot be plundered."},
+		{"installable", "This is not an installable item."},
+		{"hyperdrive", "Allows you to make hyperjumps."},
+		{"jump drive", "Lets you jump to any nearby system."}
 	};
 }
 
 
 
-OutfitInfoDisplay::OutfitInfoDisplay()
-	: descriptionHeight(0), requirementsHeight(0), attributesHeight(0), maximumHeight(0)
+OutfitInfoDisplay::OutfitInfoDisplay(const Outfit &outfit, const PlayerInfo &player, bool canSell)
 {
-}
-
-
-
-OutfitInfoDisplay::OutfitInfoDisplay(const Outfit &outfit)
-{
-	Update(outfit);
+	Update(outfit, player, canSell);
 }
 
 
 
 // Call this every time the ship changes.
-void OutfitInfoDisplay::Update(const Outfit &outfit)
+void OutfitInfoDisplay::Update(const Outfit &outfit, const PlayerInfo &player, bool canSell)
 {
-	UpdateDescription(outfit);
-	UpdateRequirements(outfit);
+	UpdateDescription(outfit.Description(), outfit.Licenses(), false);
+	UpdateRequirements(outfit, player, canSell);
 	UpdateAttributes(outfit);
 	
 	maximumHeight = max(descriptionHeight, max(requirementsHeight, attributesHeight));
@@ -106,29 +91,6 @@ void OutfitInfoDisplay::Update(const Outfit &outfit)
 
 
 
-// Get the panel width.
-int OutfitInfoDisplay::PanelWidth()
-{
-	return WIDTH;
-}
-
-
-
-// Get the height of each of the three panels.
-int OutfitInfoDisplay::MaximumHeight() const
-{
-	return maximumHeight;
-}
-
-
-
-int OutfitInfoDisplay::DescriptionHeight() const
-{
-	return descriptionHeight;
-}
-
-
-
 int OutfitInfoDisplay::RequirementsHeight() const
 {
 	return requirementsHeight;
@@ -136,21 +98,6 @@ int OutfitInfoDisplay::RequirementsHeight() const
 
 
 
-int OutfitInfoDisplay::AttributesHeight() const
-{
-	return attributesHeight;
-}
-
-
-
-// Draw each of the panels.
-void OutfitInfoDisplay::DrawDescription(const Point &topLeft) const
-{
-	description.Draw(topLeft + Point(10., 2.), *GameData::Colors().Get("medium"));
-}
-
-
-
 void OutfitInfoDisplay::DrawRequirements(const Point &topLeft) const
 {
 	Draw(topLeft, requirementLabels, requirementValues);
@@ -158,54 +105,58 @@ void OutfitInfoDisplay::DrawRequirements(const Point &topLeft) const
 
 
 
-void OutfitInfoDisplay::DrawAttributes(const Point &topLeft) const
-{
-	Draw(topLeft, attributeLabels, attributeValues);
-}
-
-
-
-void OutfitInfoDisplay::UpdateDescription(const Outfit &outfit)
-{
-	description.SetAlignment(WrappedText::JUSTIFIED);
-	description.SetWrapWidth(WIDTH - 20);
-	description.SetFont(FontSet::Get(14));
-	
-	description.Wrap(outfit.Description());
-	
-	// Pad by 10 pixels on the top and bottom.
-	descriptionHeight = description.Height() + 20;
-}
-
-
-
-void OutfitInfoDisplay::UpdateRequirements(const Outfit &outfit)
+void OutfitInfoDisplay::UpdateRequirements(const Outfit &outfit, const PlayerInfo &player, bool canSell)
 {
 	requirementLabels.clear();
 	requirementValues.clear();
 	requirementsHeight = 20;
 	
-	requirementLabels.push_back("cost:");
-	requirementValues.push_back(Format::Number(outfit.Cost()));
+	int day = player.GetDate().DaysSinceEpoch();
+	int64_t cost = outfit.Cost();
+	int64_t buyValue = player.StockDepreciation().Value(&outfit, day);
+	int64_t sellValue = player.FleetDepreciation().Value(&outfit, day);
+	
+	if(buyValue == cost)
+		requirementLabels.push_back("cost:");
+	else
+	{
+		ostringstream out;
+		out << "cost (" << (100 * buyValue) / cost << "%):";
+		requirementLabels.push_back(out.str());
+	}
+	requirementValues.push_back(Format::Number(buyValue));
 	requirementsHeight += 20;
 	
-	static const string names[] = {
+	if(canSell && sellValue != buyValue)
+	{
+		if(sellValue == cost)
+			requirementLabels.push_back("sells for:");
+		else
+		{
+			ostringstream out;
+			out << "sells for (" << (100 * sellValue) / cost << "%):";
+			requirementLabels.push_back(out.str());
+		}
+		requirementValues.push_back(Format::Number(sellValue));
+		requirementsHeight += 20;
+	}
+	
+	static const vector<string> NAMES = {
 		"outfit space needed:", "outfit space",
 		"weapon capacity needed:", "weapon capacity",
 		"engine capacity needed:", "engine capacity",
-		"guns ports needed:", "gun ports",
-		"turret mounts needed:", "turrent mounts"
+		"gun ports needed:", "gun ports",
+		"turret mounts needed:", "turret mounts"
 	};
-	static const int NAMES =  sizeof(names) / sizeof(names[0]);
-	for(int i = 0; i + 1 < NAMES; i += 2)
-		if(outfit.Get(names[i + 1]))
+	for(unsigned i = 0; i + 1 < NAMES.size(); i += 2)
+		if(outfit.Get(NAMES[i + 1]))
 		{
 			requirementLabels.push_back(string());
 			requirementValues.push_back(string());
 			requirementsHeight += 10;
 		
-			requirementLabels.push_back(names[i]);
-			requirementValues.push_back(Format::Number(-outfit.Get(names[i + 1])));
+			requirementLabels.push_back(NAMES[i]);
+			requirementValues.push_back(Format::Number(-outfit.Get(NAMES[i + 1])));
 			requirementsHeight += 20;
 		}
 }
@@ -221,21 +172,28 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 	map<string, map<string, int>> listing;
 	for(const auto &it : outfit.Attributes())
 	{
-		if(it.first == "cost" || it.first == "outfit space"
+		if(it.first == "outfit space"
 				|| it.first == "weapon capacity" || it.first == "engine capacity"
 				|| it.first == "gun ports" || it.first == "turret mounts")
 			continue;
 		
 		string value;
-		double scale = 1.;
-		if(it.first == "thrust" || it.first == "reverse thrust" || it.first == "afterburner thrust")
-			scale = 60. * 60.;
-		else if(ATTRIBUTES_TO_SCALE.find(it.first) != ATTRIBUTES_TO_SCALE.end())
-			scale = 60.;
+		auto sit = SCALE.find(it.first);
+		double scale = (sit == SCALE.end() ? 1. : sit->second);
 		
-		attributeLabels.push_back(it.first + ':');
-		attributeValues.push_back(Format::Number(it.second * scale));
-		attributesHeight += 20;
+		auto bit = BOOLEAN_ATTRIBUTES.find(it.first);
+		if(bit != BOOLEAN_ATTRIBUTES.end()) 
+		{
+			attributeLabels.push_back(bit->second);
+			attributeValues.push_back(" ");
+			attributesHeight += 20;
+		}
+		else
+		{
+			attributeLabels.push_back(it.first + ':');
+			attributeValues.push_back(Format::Number(it.second * scale));
+			attributesHeight += 20;
+		}
 	}
 	
 	if(!outfit.IsWeapon())
@@ -284,6 +242,20 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		attributesHeight += 20;
 	}
 	
+	if(outfit.SlowingDamage() && outfit.Reload())
+	{
+		attributeLabels.push_back("slowing damage / second:");
+		attributeValues.push_back(Format::Number(6000. * outfit.SlowingDamage() / outfit.Reload()));
+		attributesHeight += 20;
+	}
+	
+	if(outfit.DisruptionDamage() && outfit.Reload())
+	{
+		attributeLabels.push_back("disruption damage / second:");
+		attributeValues.push_back(Format::Number(6000. * outfit.DisruptionDamage() / outfit.Reload()));
+		attributesHeight += 20;
+	}
+	
 	if(outfit.FiringEnergy() && outfit.Reload())
 	{
 		attributeLabels.push_back("firing energy / second:");
@@ -313,6 +285,13 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		attributeValues.push_back(Format::Number(60. / outfit.Reload()));
 	attributesHeight += 20;
 	
+	double turretTurn = outfit.TurretTurn() * 60.;
+	if(turretTurn)
+	{
+		attributeLabels.push_back("turret turn rate:");
+		attributeValues.push_back(Format::Number(turretTurn));
+		attributesHeight += 20;
+	}
 	int homing = outfit.Homing();
 	if(homing)
 	{
@@ -327,16 +306,40 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		attributeValues.push_back(skill[max(0, min(4, homing))]);
 		attributesHeight += 20;
 	}
+	static const vector<string> PERCENT_NAMES = {
+		"tracking:",
+		"optical tracking:",
+		"infrared tracking:",
+		"radar tracking:",
+		"piercing:"
+	};
+	vector<double> percentValues = {
+		outfit.Tracking(),
+		outfit.OpticalTracking(),
+		outfit.InfraredTracking(),
+		outfit.RadarTracking(),
+		outfit.Piercing()
+	};
+	for(unsigned i = 0; i < PERCENT_NAMES.size(); ++i)
+		if(percentValues[i])
+		{
+			int percent = 100. * percentValues[i] + .5;
+			attributeLabels.push_back(PERCENT_NAMES[i]);
+			attributeValues.push_back(Format::Number(percent) + "%");
+			attributesHeight += 20;
+		}
 	
 	attributeLabels.push_back(string());
 	attributeValues.push_back(string());
 	attributesHeight += 10;
 	
-	static const string names[] = {
-		"shield damage / shot: ",
-		"hull damage / shot: ",
-		"heat damage / shot: ",
-		"ion damage / shot: ",
+	static const vector<string> OTHER_NAMES = {
+		"shield damage / shot:",
+		"hull damage / shot:",
+		"heat damage / shot:",
+		"ion damage / shot:",
+		"slowing damage / shot:",
+		"disruption damage / shot:",
 		"firing energy / shot:",
 		"firing heat / shot:",
 		"firing fuel / shot:",
@@ -345,11 +348,13 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		"missile strength:",
 		"anti-missile:",
 	};
-	double values[] = {
+	vector<double> values = {
 		outfit.ShieldDamage(),
 		outfit.HullDamage(),
 		outfit.HeatDamage(),
 		outfit.IonDamage() * 100.,
+		outfit.SlowingDamage() * 100.,
+		outfit.DisruptionDamage() * 100.,
 		outfit.FiringEnergy(),
 		outfit.FiringHeat(),
 		outfit.FiringFuel(),
@@ -358,11 +363,10 @@ void OutfitInfoDisplay::UpdateAttributes(const Outfit &outfit)
 		static_cast<double>(outfit.MissileStrength()),
 		static_cast<double>(outfit.AntiMissile())
 	};
-	static const int NAMES = sizeof(names) / sizeof(names[0]);
-	for(int i = (isContinuous ? 7 : 0); i < NAMES; ++i)
+	for(unsigned i = (isContinuous ? 9 : 0); i < OTHER_NAMES.size(); ++i)
 		if(values[i])
 		{
-			attributeLabels.push_back(names[i]);
+			attributeLabels.push_back(OTHER_NAMES[i]);
 			attributeValues.push_back(Format::Number(values[i]));
 			attributesHeight += 20;
 		}
