@@ -22,14 +22,12 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #include "Format.h"
 #include "GameData.h"
 #include "Outfit.h"
-#include "OutfitInfoDisplay.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
 #include "Point.h"
-#include "Preferences.h"
 #include "Screen.h"
 #include "Ship.h"
-#include "ShipInfoDisplay.h"
+#include "Sprite.h"
 #include "SpriteSet.h"
 #include "SpriteShader.h"
 #include "UI.h"
@@ -41,18 +39,6 @@ class Sprite;
 using namespace std;
 
 namespace {
-	static const vector<string> CATEGORIES = {
-		"Guns",
-		"Turrets",
-		"Secondary Weapons",
-		"Ammunition",
-		"Systems",
-		"Power",
-		"Engines",
-		"Hand to Hand",
-		"Special"
-	};
-	
 	string Tons(int tons)
 	{
 		return to_string(tons) + (tons == 1 ? " ton" : " tons");
@@ -62,7 +48,7 @@ namespace {
 
 
 OutfitterPanel::OutfitterPanel(PlayerInfo &player)
-	: ShopPanel(player, CATEGORIES), available(player.SoldOutfits())
+	: ShopPanel(player, true)
 {
 	for(const pair<string, Outfit> &it : GameData::Outfits())
 		catalog[it.second.Category()].insert(it.first);
@@ -76,23 +62,8 @@ OutfitterPanel::OutfitterPanel(PlayerInfo &player)
 void OutfitterPanel::Step()
 {
 	CheckRefill();
-	if(!Preferences::Has("help: outfitter"))
-	{
-		Preferences::Set("help: outfitter");
-		GetUI()->Push(new Dialog(
-			"Here, you can buy new equipment for your ship. "
-			"Your ship has a limited amount of \"outfit space,\" "
-			"and most outfits use up some of that space.\n"
-			"\tSome types of outfits have other requirements as well. "
-			"For example, only some of your outfit space can be used for engines or weapons; "
-			"this is your ship's \"engine capacity\" and \"weapon capacity.\" "
-			"Guns and missile launchers also require a free \"gun port,\" "
-			"and turrets require a free \"turret mount.\" "
-			"Also, missiles can only be bought if you have the right launcher installed.\n"
-			"\tUse your scroll wheel, or click and drag, to scroll the view.\n"
-			"\tAs in the trading panel, you can hold down Shift or Control "
-			"to buy 5 or 20 copies of an outfit at once, or both keys to buy 100."));
-	}
+	DoHelp("outfitter");
+	ShopPanel::Step();
 }
 
 
@@ -104,34 +75,41 @@ int OutfitterPanel::TileSize() const
 
 
 
-int OutfitterPanel::DrawPlayerShipInfo(const Point &point) const
+int OutfitterPanel::DrawPlayerShipInfo(const Point &point)
 {
-	ShipInfoDisplay info(*playerShip);
-	info.DrawAttributes(point);
+	shipInfo.Update(*playerShip, player.FleetDepreciation(), day);
+	shipInfo.DrawAttributes(point);
 	
-	return info.AttributesHeight();
+	return shipInfo.AttributesHeight();
 }
 
 
 
-bool OutfitterPanel::DrawItem(const string &name, const Point &point, int scrollY) const
+bool OutfitterPanel::HasItem(const string &name) const
 {
 	const Outfit *outfit = GameData::Outfits().Get(name);
-	bool hasOutfit = (outfitter.Has(outfit) || available[outfit] || player.Cargo().Get(outfit));
-	if(!hasOutfit)
-		for(const Ship *ship : playerShips)
-			if(ship->OutfitCount(outfit))
-			{
-				hasOutfit = true;
-				break;
-			}
-	if(!hasOutfit)
-		return false;
-	
-	zones.emplace_back(point.X(), point.Y(), OUTFIT_SIZE / 2, OUTFIT_SIZE / 2, outfit, scrollY);
-	if(point.Y() + OUTFIT_SIZE / 2 < Screen::Top() || point.Y() - OUTFIT_SIZE / 2 > Screen::Bottom())
+	if((outfitter.Has(outfit) || player.Stock(outfit) > 0) && showForSale)
 		return true;
 	
+	if(player.Cargo().Get(outfit) && (!playerShip || showForSale))
+		return true;
+	
+	for(const Ship *ship : playerShips)
+		if(ship->OutfitCount(outfit))
+			return true;
+	
+	return false;
+}
+
+
+
+void OutfitterPanel::DrawItem(const string &name, const Point &point, int scrollY)
+{
+	const Outfit *outfit = GameData::Outfits().Get(name);
+	zones.emplace_back(point, Point(OUTFIT_SIZE, OUTFIT_SIZE), outfit, scrollY);
+	if(point.Y() + OUTFIT_SIZE / 2 < Screen::Top() || point.Y() - OUTFIT_SIZE / 2 > Screen::Bottom())
+		return;
+	
 	bool isSelected = (outfit == selectedOutfit);
 	bool isOwned = playerShip && playerShip->OutfitCount(outfit);
 	DrawOutfit(*outfit, point, isSelected, isOwned);
@@ -170,24 +148,29 @@ bool OutfitterPanel::DrawItem(const string &name, const Point &point, int scroll
 			font.Draw(label, labelPos, bright);
 		}
 	}
-	if(player.Cargo().Get(outfit))
+	// Don't show the "in stock" amount if the outfit has an unlimited stock or
+	// if it is not something that you can buy.
+	int stock = 0;
+	if(!outfitter.Has(outfit) && outfit->Get("installable") >= 0.)
+		stock = max(0, player.Stock(outfit));
+	int cargo = player.Cargo().Get(outfit);
+	
+	string message;
+	if(cargo && stock)
+		message = "in cargo: " + to_string(cargo) + ", in stock: " + to_string(stock);
+	else if(cargo)
+		message = "in cargo: " + to_string(cargo);
+	else if(stock)
+		message = "in stock: " + to_string(stock);
+	else if(!outfitter.Has(outfit))
+		message = "(not sold here)";
+	if(!message.empty())
 	{
-		string count = "in cargo: " + to_string(player.Cargo().Get(outfit));
-		Point pos = point + Point(
-			OUTFIT_SIZE / 2 - 20 - font.Width(count),
-			OUTFIT_SIZE / 2 - 24);
-		font.Draw(count, pos, bright);
-	}
-	else if(!outfitter.Has(outfit) && !available[outfit])
-	{
-		static const string message = "(not sold here)";
 		Point pos = point + Point(
 			OUTFIT_SIZE / 2 - 20 - font.Width(message),
 			OUTFIT_SIZE / 2 - 24);
 		font.Draw(message, pos, bright);
 	}
-	
-	return true;
 }
 
 
@@ -201,46 +184,60 @@ int OutfitterPanel::DividerOffset() const
 
 int OutfitterPanel::DetailWidth() const
 {
-	return 3 * OutfitInfoDisplay::PanelWidth();
+	return 3 * outfitInfo.PanelWidth();
 }
 
 
 
-int OutfitterPanel::DrawDetails(const Point &center) const
+int OutfitterPanel::DrawDetails(const Point &center)
 {
-	OutfitInfoDisplay info(*selectedOutfit);
-	Point offset(info.PanelWidth(), 0.);
+	if(!selectedOutfit)
+		return 0;
 	
-	info.DrawDescription(center - offset * 1.5);
-	info.DrawRequirements(center - offset * .5);
-	info.DrawAttributes(center + offset * .5);
+	outfitInfo.Update(*selectedOutfit, player, CanSell());
+	Point offset(outfitInfo.PanelWidth(), 0.);
 	
-	return info.MaximumHeight() + 40;
+	outfitInfo.DrawDescription(center - offset * 1.5 - Point(0., 10.));
+	outfitInfo.DrawRequirements(center - offset * .5 - Point(0., 10.));
+	outfitInfo.DrawAttributes(center + offset * .5 - Point(0., 10.));
+	
+	return outfitInfo.MaximumHeight();
 }
 
 
 
 bool OutfitterPanel::CanBuy() const
 {
-	if(!planet || !selectedOutfit || !playerShip)
+	if(!planet || !selectedOutfit)
 		return false;
 	
-	// If you have this in your cargo hold, installing it is free.
-	if(player.Cargo().Get(selectedOutfit))
-		return true;
-	
-	if(!(outfitter.Has(selectedOutfit) || available[selectedOutfit]))
+	bool isInCargo = player.Cargo().Get(selectedOutfit) && playerShip;
+	if(!(outfitter.Has(selectedOutfit) || player.Stock(selectedOutfit) > 0 || isInCargo))
 		return false;
 	
 	int mapSize = selectedOutfit->Get("map");
 	if(mapSize > 0 && HasMapped(mapSize))
 		return false;
 	
+	// Determine what you will have to pay to buy this outfit.
+	int64_t cost = player.StockDepreciation().Value(selectedOutfit, day);
+	// Check that the player has any necessary licenses.
+	int64_t licenseCost = LicenseCost(selectedOutfit);
+	if(licenseCost < 0)
+		return false;
+	cost += licenseCost;
+	// If you have this in your cargo hold, installing it is free.
+	if(cost > player.Accounts().Credits() && !isInCargo)
+		return false;
+	
 	if(HasLicense(selectedOutfit->Name()))
 		return false;
 	
-	if(selectedOutfit->Cost() > player.Accounts().Credits())
-		return false;
+	if(!playerShip)
+	{
+		double mass = selectedOutfit->Get("mass");
+		return (!mass || player.Cargo().Free() >= mass);
+	}
 	
 	for(const Ship *ship : playerShips)
 		if(ShipCanBuy(ship, selectedOutfit))
@@ -253,6 +250,15 @@ bool OutfitterPanel::CanBuy() const
 
 void OutfitterPanel::Buy()
 {
+	int64_t licenseCost = LicenseCost(selectedOutfit);
+	if(licenseCost)
+	{
+		player.Accounts().AddCredits(-licenseCost);
+		for(const string &licenseName : selectedOutfit->Licenses())
+			if(!player.GetCondition("license: " + licenseName))
+				player.Conditions()["license: " + licenseName] = true;
+	}
+	
 	int modifier = Modifier();
 	for(int i = 0; i < modifier && CanBuy(); ++i)
 	{
@@ -263,14 +269,15 @@ void OutfitterPanel::Buy()
 			if(!HasMapped(mapSize))
 			{
 				DistanceMap distance(player.GetSystem(), mapSize);
-				for(const auto &it : distance.Distances())
-					if(!player.HasVisited(it.first))
-						player.Visit(it.first);
-				player.Accounts().AddCredits(-selectedOutfit->Cost());
+				for(const System *system : distance.Systems())
+					if(!player.HasVisited(system))
+						player.Visit(system);
+				int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
+				player.Accounts().AddCredits(-price);
 			}
 			return;
 		}
-	
+		
 		// Special case: licenses.
 		if(IsLicense(selectedOutfit->Name()))
 		{
@@ -278,11 +285,21 @@ void OutfitterPanel::Buy()
 			if(entry <= 0)
 			{
 				entry = true;
-				player.Accounts().AddCredits(-selectedOutfit->Cost());
+				int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
+				player.Accounts().AddCredits(-price);
 			}
 			return;
 		}
-	
+		
+		if(!playerShip)
+		{
+			player.Cargo().Add(selectedOutfit);
+			int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
+			player.Accounts().AddCredits(-price);
+			player.AddStock(selectedOutfit, -1);
+			continue;
+		}
+		
 		// Find the ships with the fewest number of these outfits.
 		vector<Ship *> shipsToOutfit;
 		int fewest = numeric_limits<int>::max();
@@ -307,19 +324,20 @@ void OutfitterPanel::Buy()
 				return;
 		
 			if(player.Cargo().Get(selectedOutfit))
-				player.Cargo().Transfer(selectedOutfit, 1);
-			else if(!available[selectedOutfit] && !outfitter.Has(selectedOutfit))
+				player.Cargo().Remove(selectedOutfit);
+			else if(!(player.Stock(selectedOutfit) > 0 || outfitter.Has(selectedOutfit)))
 				break;
 			else
 			{
-				player.Accounts().AddCredits(-selectedOutfit->Cost());
-				--available[selectedOutfit];
+				int64_t price = player.StockDepreciation().Value(selectedOutfit, day);
+				player.Accounts().AddCredits(-price);
+				player.AddStock(selectedOutfit, -1);
 			}
 			ship->AddOutfit(selectedOutfit, 1);
 			int required = selectedOutfit->Get("required crew");
 			if(required && ship->Crew() + required <= static_cast<int>(ship->Attributes().Get("bunks")))
 				ship->AddCrew(required);
-			ship->Recharge(true);
+			ship->Recharge();
 		}
 	}
 }
@@ -328,28 +346,60 @@ void OutfitterPanel::Buy()
 
 void OutfitterPanel::FailBuy() const
 {
-	if(!selectedOutfit || !playerShip)
+	if(!selectedOutfit)
 		return;
 	
-	int64_t cost = selectedOutfit->Cost();
+	int64_t cost = player.StockDepreciation().Value(selectedOutfit, day);
 	int64_t credits = player.Accounts().Credits();
 	bool isInCargo = player.Cargo().Get(selectedOutfit);
 	if(!isInCargo && cost > credits)
 	{
 		GetUI()->Push(new Dialog("You cannot buy this outfit, because it costs "
-			+ to_string(cost) + " credits, and you only have "
-			+ to_string(credits) + "."));
+			+ Format::Number(cost) + " credits, and you only have "
+			+ Format::Number(credits) + "."));
+		return;
+	}
+	// Check that the player has any necessary licenses.
+	int64_t licenseCost = LicenseCost(selectedOutfit);
+	if(licenseCost < 0)
+	{
+		GetUI()->Push(new Dialog(
+			"You cannot buy this outfit, because it requires a license that you don't have."));
+		return;
+	}
+	if(!isInCargo && cost + licenseCost > credits)
+	{
+		GetUI()->Push(new Dialog(
+			"You don't have enough money to buy this outfit, because it will cost you an extra "
+			+ Format::Number(licenseCost) + " credits to buy the necessary licenses."));
 		return;
 	}
 	
-	if(!(outfitter.Has(selectedOutfit) || available[selectedOutfit] || isInCargo))
+	if(!(outfitter.Has(selectedOutfit) || player.Stock(selectedOutfit) > 0 || isInCargo))
 	{
 		GetUI()->Push(new Dialog("You cannot buy this outfit here. "
 			"It is being shown in the list because you have one installed in your ship, "
-			"but this " + planet->Noun() + " does not sell additional copies of it."));
+			"but this " + planet->Noun() + " does not sell them."));
 		return;
 	}
 	
+	if(selectedOutfit->Get("map"))
+	{
+		GetUI()->Push(new Dialog("You have already mapped all the systems shown by this map, "
+			"so there is no reason to buy another."));
+		return;
+	}
+	
+	if(HasLicense(selectedOutfit->Name()))
+	{
+		GetUI()->Push(new Dialog("You already have one of these licenses, "
+			"so there is no reason to buy another."));
+		return;
+	}
+	
+	if(!playerShip)
+		return;
+	
 	double outfitNeeded = -selectedOutfit->Get("outfit space");
 	double outfitSpace = playerShip->Attributes().Get("outfit space");
 	if(outfitNeeded > outfitSpace)
@@ -390,7 +440,7 @@ void OutfitterPanel::FailBuy() const
 				"You cannot install it without first installing the appropriate weapon."));
 		else
 			GetUI()->Push(new Dialog("You already have the maximum amount of ammunition for this weapon. "
-				"If you want to install more ammunition, you must first install another copy of the weapon."));
+				"If you want to install more ammunition, you must first install another of these weapons."));
 		return;
 	}
 	
@@ -412,6 +462,12 @@ void OutfitterPanel::FailBuy() const
 		return;
 	}
 	
+	if(selectedOutfit->Get("installable") < 0.)
+	{
+		GetUI()->Push(new Dialog("This item is not an outfit that can be installed in a ship."));
+		return;
+	}
+	
 	if(!playerShip->Attributes().CanAdd(*selectedOutfit, 1))
 	{
 		GetUI()->Push(new Dialog("You cannot install this outfit in your ship, "
@@ -420,20 +476,6 @@ void OutfitterPanel::FailBuy() const
 			"or has a constant energy draw greater than what your ship produces."));
 		return;
 	}
-	
-	if(selectedOutfit->Get("map"))
-	{
-		GetUI()->Push(new Dialog("You have already mapped all the systems shown by this map, "
-			"so there is no reason to buy another."));
-		return;
-	}
-	
-	if(HasLicense(selectedOutfit->Name()))
-	{
-		GetUI()->Push(new Dialog("You already have one of these licenses, "
-			"so there is no reason to buy another."));
-		return;
-	}
 }
 
 
@@ -459,9 +501,10 @@ void OutfitterPanel::Sell()
 {
 	if(player.Cargo().Get(selectedOutfit))
 	{
-		player.Cargo().Transfer(selectedOutfit, 1);
-		player.Accounts().AddCredits(selectedOutfit->Cost());
-		++available[selectedOutfit];
+		player.Cargo().Remove(selectedOutfit);
+		int64_t price = player.FleetDepreciation().Value(selectedOutfit, day);
+		player.Accounts().AddCredits(price);
+		player.AddStock(selectedOutfit, 1);
 	}
 	else
 	{
@@ -488,9 +531,10 @@ void OutfitterPanel::Sell()
 			ship->AddOutfit(selectedOutfit, -1);
 			if(selectedOutfit->Get("required crew"))
 				ship->AddCrew(-selectedOutfit->Get("required crew"));
-			ship->Recharge(true);
-			player.Accounts().AddCredits(selectedOutfit->Cost());
-			++available[selectedOutfit];
+			ship->Recharge();
+			int64_t price = player.FleetDepreciation().Value(selectedOutfit, day);
+			player.Accounts().AddCredits(price);
+			player.AddStock(selectedOutfit, 1);
 			
 			const Outfit *ammo = selectedOutfit->Ammo();
 			if(ammo && ship->OutfitCount(ammo))
@@ -499,13 +543,14 @@ void OutfitterPanel::Sell()
 				int mustSell = 0;
 				for(const auto &it : ship->Attributes().Attributes())
 					if(it.second < 0.)
-						mustSell = max(mustSell, static_cast<int>(it.second / ammo->Get(it.first)));
+						mustSell = max<int>(mustSell, it.second / ammo->Get(it.first));
 				
 				if(mustSell)
 				{
 					ship->AddOutfit(ammo, -mustSell);
-					player.Accounts().AddCredits(ammo->Cost() * mustSell);
-					available[ammo] += mustSell;
+					int64_t price = player.FleetDepreciation().Value(ammo, day, mustSell);
+					player.Accounts().AddCredits(price);
+					player.AddStock(ammo, mustSell);
 				}
 			}
 		}
@@ -524,7 +569,7 @@ void OutfitterPanel::FailSell() const
 		GetUI()->Push(new Dialog("You cannot sell licenses. Once you buy one, it is yours permanently."));
 	else
 	{
-		bool hasOutfit = false;
+		bool hasOutfit = player.Cargo().Get(selectedOutfit);
 		for(const Ship *ship : playerShips)
 			if(ship->OutfitCount(selectedOutfit))
 			{
@@ -567,40 +612,57 @@ bool OutfitterPanel::FlightCheck()
 	for(const shared_ptr<Ship> &ship : player.Ships())
 	{
 		// Skip any ships that are "absent" for whatever reason.
-		if(ship->GetSystem() != player.GetSystem())
+		if(ship->GetSystem() != player.GetSystem() || ship->IsDisabled())
 			continue;
 		
 		const Outfit &attributes = ship->Attributes();
-		double energy = attributes.Get("energy generation") + attributes.Get("energy capacity");
-		if(!attributes.Get("thrust"))
+		double energy = attributes.Get("energy generation") - attributes.Get("energy consumption");
+		energy += attributes.Get("solar collection");
+		energy += attributes.Get("energy capacity");
+		if(energy < 0.)
 		{
 			GetUI()->Push(new ConversationPanel(player,
-				*GameData::Conversations().Get("flight check: no thrusters")));
+				*GameData::Conversations().Get("flight check: no energy"), nullptr, ship.get()));
+			return false;			
+		}
+		if(!attributes.Get("thrust") && !attributes.Get("reverse thrust")
+				&& !attributes.Get("afterburner thrust"))
+		{
+			GetUI()->Push(new ConversationPanel(player,
+				*GameData::Conversations().Get("flight check: no thrusters"), nullptr, ship.get()));
 			return false;
 		}
 		if(attributes.Get("thrusting energy") > energy)
 		{
 			GetUI()->Push(new ConversationPanel(player,
-				*GameData::Conversations().Get("flight check: no thruster energy")));
+				*GameData::Conversations().Get("flight check: no thruster energy"), nullptr, ship.get()));
 			return false;
 		}
 		if(!attributes.Get("turn"))
 		{
 			GetUI()->Push(new ConversationPanel(player,
-				*GameData::Conversations().Get("flight check: no steering")));
+				*GameData::Conversations().Get("flight check: no steering"), nullptr, ship.get()));
 			return false;
 		}
 		if(attributes.Get("turning energy") > energy)
 		{
 			GetUI()->Push(new ConversationPanel(player,
-				*GameData::Conversations().Get("flight check: no steering energy")));
+				*GameData::Conversations().Get("flight check: no steering energy"), nullptr, ship.get()));
 			return false;
 		}
-		double maxHeat = .1 * ship->Mass() * attributes.Get("heat dissipation");
-		if(attributes.Get("heat generation") - attributes.Get("cooling") > maxHeat)
+		// Check energy balance with solar collection at infinite distance from a star (only 20% of
+		// maximum solar collection).
+		energy -= .8 * attributes.Get("solar collection");
+		if(attributes.Get("thrusting energy") > energy || attributes.Get("turning energy") > energy)
 		{
 			GetUI()->Push(new ConversationPanel(player,
-				*GameData::Conversations().Get("flight check: overheating")));
+				*GameData::Conversations().Get("flight check: solar power warning"), nullptr, ship.get()));
+			return true;
+		}			
+		if(ship->IdleHeat() >= 100. * ship->Mass())
+		{
+			GetUI()->Push(new ConversationPanel(player,
+				*GameData::Conversations().Get("flight check: overheating"), nullptr, ship.get()));
 			return false;
 		}
 	}
@@ -609,9 +671,52 @@ bool OutfitterPanel::FlightCheck()
 
 
 
+void OutfitterPanel::DrawKey()
+{
+	const Sprite *back = SpriteSet::Get("ui/outfitter key");
+	SpriteShader::Draw(back, Screen::BottomLeft() + .5 * Point(back->Width(), -back->Height()));
+	
+	Font font = FontSet::Get(14);
+	Color color[2] = {*GameData::Colors().Get("medium"), *GameData::Colors().Get("bright")};
+	const Sprite *box[2] = {SpriteSet::Get("ui/unchecked"), SpriteSet::Get("ui/checked")};
+	
+	Point pos = Screen::BottomLeft() + Point(10., -30.);
+	Point off = Point(10., -.5 * font.Height());
+	SpriteShader::Draw(box[showForSale], pos);
+	font.Draw("Show outfits for sale", pos + off, color[showForSale]);
+	AddZone(Rectangle(pos + Point(80., 0.), Point(180., 20.)), [this](){ showForSale = !showForSale; });
+	
+	bool showCargo = !playerShip;
+	pos.Y() += 20.;
+	SpriteShader::Draw(box[showCargo], pos);
+	font.Draw("Show outfits in cargo", pos + off, color[showCargo]);
+	AddZone(Rectangle(pos + Point(80., 0.), Point(180., 20.)), [this](){
+		if(playerShip)
+		{
+			previousShip = playerShip;
+			playerShip = nullptr;
+			previousShips = playerShips;
+			playerShips.clear();
+		}
+		else if(previousShip)
+		{
+			playerShip = previousShip;
+			playerShips = previousShips;
+		}
+		else
+		{
+			playerShip = player.Flagship();
+			if(playerShip)
+				playerShips.insert(playerShip);
+		}
+	});
+}
+
+
+
 bool OutfitterPanel::ShipCanBuy(const Ship *ship, const Outfit *outfit)
 {
-	return ship->Attributes().CanAdd(*outfit, 1);
+	return (ship->Attributes().CanAdd(*outfit, 1) > 0);
 }
 
 
@@ -636,7 +741,7 @@ bool OutfitterPanel::ShipCanSell(const Ship *ship, const Outfit *outfit)
 }
 
 
-	
+
 void OutfitterPanel::DrawOutfit(const Outfit &outfit, const Point &center, bool isSelected, bool isOwned)
 {
 	const Sprite *thumbnail = outfit.Thumbnail();
@@ -657,8 +762,8 @@ void OutfitterPanel::DrawOutfit(const Outfit &outfit, const Point &center, bool
 bool OutfitterPanel::HasMapped(int mapSize) const
 {
 	DistanceMap distance(player.GetSystem(), mapSize);
-	for(const auto &it : distance.Distances())
-		if(!player.HasVisited(it.first))
+	for(const System *system : distance.Systems())
+		if(!player.HasVisited(system))
 			return false;
 	
 	return true;
@@ -700,8 +805,8 @@ void OutfitterPanel::CheckRefill()
 		return;
 	checkedRefill = true;
 	
-	int64_t cost = 0;
 	int count = 0;
+	map<const Outfit *, int> needed;
 	for(const auto &ship : player.Ships())
 	{
 		if(ship->GetSystem() != player.GetSystem() || ship->IsDisabled())
@@ -715,19 +820,27 @@ void OutfitterPanel::CheckRefill()
 		
 		for(const Outfit *outfit : toRefill)
 		{
-			int needed = ship->Attributes().CanAdd(*outfit, 1000000);
-			if(!outfitter.Has(outfit))
-				needed = min(needed, player.Cargo().Get(outfit) + available[selectedOutfit]);
-			cost += needed * outfit->Cost();
+			int amount = ship->Attributes().CanAdd(*outfit, 1000000);
+			if(amount > 0 && (outfitter.Has(outfit) || player.Stock(outfit) > 0 || player.Cargo().Get(outfit)))
+				needed[outfit] += amount;
 		}
 	}
 	
-	if(cost && cost < player.Accounts().Credits())
+	int64_t cost = 0;
+	for(auto &it : needed)
 	{
-		string message = (count == 1) ?
-			"Do you want to reload all the ammunition for your ship? It will cost " :
-			"Do you want to reload all the ammunition for your ships? It will cost ";
-		message += Format::Number(cost) + " credits.";
+		// Don't count cost of anything installed from cargo.
+		it.second = max(0, it.second - player.Cargo().Get(it.first));
+		if(!outfitter.Has(it.first))
+			it.second = min(it.second, max(0, player.Stock(it.first)));
+		cost += player.StockDepreciation().Value(it.first, day, it.second);
+	}
+	if(!needed.empty() && cost < player.Accounts().Credits())
+	{
+		string message = "Do you want to reload all the ammunition for your ship";
+		message += (count == 1) ? "?" : "s?";
+		if(cost)
+			message += " It will cost " + Format::Number(cost) + " credits.";
 		GetUI()->Push(new Dialog(this, &OutfitterPanel::Refill, message));
 	}
 }
@@ -738,7 +851,7 @@ void OutfitterPanel::Refill()
 {
 	for(const auto &ship : player.Ships())
 	{
-		if(ship->GetSystem() != player.GetSystem())
+		if(ship->GetSystem() != player.GetSystem() || ship->IsDisabled())
 			continue;
 		
 		set<const Outfit *> toRefill;
@@ -749,16 +862,17 @@ void OutfitterPanel::Refill()
 		// This is slower than just calculating the proper number to add, but
 		// that does not matter because this is not so time-consuming anyways.
 		for(const Outfit *outfit : toRefill)
-			while(ship->Attributes().CanAdd(*outfit))
+			while(ship->Attributes().CanAdd(*outfit) > 0)
 			{
 				if(player.Cargo().Get(outfit))
-					player.Cargo().Transfer(outfit, 1);
-				else if(!available[outfit] && !outfitter.Has(outfit))
+					player.Cargo().Remove(outfit);
+				else if(!(player.Stock(outfit) > 0 || outfitter.Has(outfit)))
 					break;
 				else
 				{
-					player.Accounts().AddCredits(-outfit->Cost());
-					--available[outfit];
+					int64_t price = player.StockDepreciation().Value(outfit, day);
+					player.Accounts().AddCredits(-price);
+					player.AddStock(outfit, -1);
 				}
 				ship->AddOutfit(outfit, 1);
 			}
