@@ -26,7 +26,7 @@ class OutlineShader {
 public:
 	static void Init();
 	
-	static void Draw(const Sprite *sprite, const Point &pos, const Point &size, const Color &color, const Point &unit = Point(0., -1.));
+	static void Draw(const Sprite *sprite, const Point &pos, const Point &size, const Color &color, const Point &unit = Point(0., -1.), int frame = 0);
 };
 
 
