@@ -23,6 +23,7 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 class DataNode;
 class Fleet;
+class Government;
 class Outfit;
 class PlayerInfo;
 class Ship;
@@ -48,6 +49,8 @@ public:
 	const std::string &Description() const;
 	// Get the landscape sprite.
 	const Sprite *Landscape() const;
+	// Get the name of the ambient audio to play on this planet.
+	const std::string &MusicName() const;
 	
 	// Get the list of "attributes" of the planet.
 	const std::set<std::string> &Attributes() const;
@@ -74,6 +77,8 @@ public:
 	// Get the list of outfits available from the outfitter.
 	const Sale<Outfit> &Outfitter() const;
 	
+	// Get this planet's government. If not set, returns the system's government.
+	const Government *GetGovernment() const;
 	// You need this good a reputation with this system's government to land here.
 	double RequiredReputation() const;
 	// This is what fraction of your fleet's value you must pay as a bribe in
@@ -86,6 +91,9 @@ public:
 	// Set or get what system this planet is in. This is so that missions, for
 	// example, can just hold a planet pointer instead of a system as well.
 	const System *GetSystem() const;
+	// Check if this planet is in the given system. Note that wormholes may be
+	// in more than one system.
+	bool IsInSystem(const System *system) const;
 	void SetSystem(const System *system);
 	// Remove the given system from the list of systems this planet is in. This
 	// must be done when game events rearrange the planets in a system.
@@ -96,6 +104,9 @@ public:
 	const System *WormholeSource(const System *from) const;
 	const System *WormholeDestination(const System *from) const;
 	
+	// Check if the given ship has all the attributes necessary to allow it to
+	// land on this planet.
+	bool IsAccessible(const Ship *ship) const;
 	// Below are convenience functions which access the game state in Politics,
 	// but do so with a less convoluted syntax:
 	bool CanLand(const Ship &ship) const;
@@ -114,16 +125,18 @@ private:
 	std::string description;
 	std::string spaceport;
 	const Sprite *landscape = nullptr;
+	std::string music;
 	
 	std::set<std::string> attributes;
 	
-	std::vector<const Sale<Ship> *> shipSales;
-	std::vector<const Sale<Outfit> *> outfitSales;
+	std::set<const Sale<Ship> *> shipSales;
+	std::set<const Sale<Outfit> *> outfitSales;
 	// The lists above will be converted into actual ship lists when they are
 	// first asked for:
 	mutable Sale<Ship> shipyard;
 	mutable Sale<Outfit> outfitter;
 	
+	const Government *government = nullptr;
 	double requiredReputation = 0.;
 	double bribe = 0.01;
 	double security = .25;
