@@ -24,6 +24,20 @@ public:
 	
 	static bool Has(const std::string &name);
 	static void Set(const std::string &name, bool on = true);
+	
+	// Toogle the ammo usage preferences, cycling between "never," "frugally,"
+	// and "always."
+	static void ToggleAmmoUsage();
+	static std::string AmmoUsage();
+	
+	// Scroll speed preference.
+	static int ScrollSpeed();
+	static void SetScrollSpeed(int speed);
+	
+	// View zoom.
+	static double ViewZoom();
+	static bool ZoomViewIn();
+	static bool ZoomViewOut();
 };
 
 
