@@ -24,10 +24,12 @@ class RingShader {
 public:
 	static void Init();
 	
-	static void Draw(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f);
+	static void Draw(const Point &pos, float out, float in, const Color &color);
+	static void Draw(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f, float startAngle = 0.f);
 	
 	static void Bind();
-	static void Add(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f);
+	static void Add(const Point &pos, float out, float in, const Color &color);
+	static void Add(const Point &pos, float radius, float width, float fraction, const Color &color, float dash = 0.f, float startAngle = 0.f);
 	static void Unbind();
 };
 
