@@ -41,10 +41,16 @@ void Sale<Item>::Load(const DataNode &node, const Set<Item> &items)
 {
 	for(const DataNode &child : node)
 	{
-		if(child.Token(0) == "clear")
+		const std::string &token = child.Token(0);
+		bool remove = (token == "clear" || token == "remove");
+		if(remove && child.Size() == 1)
 			this->clear();
+		else if(remove && child.Size() >= 2)
+			this->erase(items.Get(child.Token(1)));
+		else if(token == "add" && child.Size() >= 2)
+			this->insert(items.Get(child.Token(1)));
 		else
-			this->insert(items.Get(child.Token(0)));
+			this->insert(items.Get(token));
 	}
 }
 
@@ -61,7 +67,7 @@ void Sale<Item>::Add(const Sale<Item> &other)
 template <class Item>
 bool Sale<Item>::Has(const Item *item) const
 {
-	return this->find(item) != this->end();
+	return this->count(item);
 }
 
 
