@@ -12,38 +12,67 @@ PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 
 #include "ShipyardPanel.h"
 
+#include "Color.h"
 #include "Dialog.h"
+#include "Font.h"
+#include "FontSet.h"
 #include "Format.h"
 #include "GameData.h"
+#include "Phrase.h"
 #include "Planet.h"
 #include "PlayerInfo.h"
 #include "Point.h"
 #include "Screen.h"
 #include "Ship.h"
-#include "ShipInfoDisplay.h"
+#include "SpriteSet.h"
+#include "SpriteShader.h"
 #include "System.h"
 #include "UI.h"
 
 using namespace std;
 
 namespace {
-	static const vector<string> CATEGORIES = {
-		"Transport",
-		"Light Freighter",
-		"Heavy Freighter",
-		"Interceptor",
-		"Light Warship",
-		"Medium Warship",
-		"Heavy Warship",
-		"Fighter",
-		"Drone"
+	// The name entry dialog should include a "Random" button to choose a random
+	// name using the civilian ship name generator.
+	class NameDialog : public Dialog {
+	public:
+		NameDialog(ShipyardPanel *panel, void (ShipyardPanel::*fun)(const string &), const string &message)
+			: Dialog(panel, fun, message) {}
+		
+		virtual void Draw() override
+		{
+			Dialog::Draw();
+			
+			randomPos = cancelPos - Point(80., 0.);
+			SpriteShader::Draw(SpriteSet::Get("ui/dialog cancel"), randomPos);
+
+			const Font &font = FontSet::Get(14);
+			static const string label = "Random";
+			Point labelPos = randomPos - .5 * Point(font.Width(label), font.Height());
+			font.Draw(label, labelPos, *GameData::Colors().Get("medium"));
+		}
+		
+	protected:
+		virtual bool Click(int x, int y, int clicks) override
+		{
+			Point off = Point(x, y) - randomPos;
+			if(fabs(off.X()) < 40. && fabs(off.Y()) < 20.)
+			{
+				input = GameData::Phrases().Get("civilian")->Get();
+				return true;
+			}
+			return Dialog::Click(x, y, clicks);
+		}
+		
+	private:
+		Point randomPos;
 	};
 }
 
 
 
 ShipyardPanel::ShipyardPanel(PlayerInfo &player)
-	: ShopPanel(player, CATEGORIES)
+	: ShopPanel(player, false), modifier(0)
 {
 	for(const auto &it : GameData::Ships())
 		catalog[it.second.Attributes().Category()].insert(it.first);
@@ -61,29 +90,33 @@ int ShipyardPanel::TileSize() const
 
 
 
-int ShipyardPanel::DrawPlayerShipInfo(const Point &point) const
+int ShipyardPanel::DrawPlayerShipInfo(const Point &point)
 {
-	ShipInfoDisplay info(*playerShip, player.GetSystem()->GetGovernment());
-	info.DrawSale(point);
+	shipInfo.Update(*playerShip, player.FleetDepreciation(), player.GetDate().DaysSinceEpoch());
+	shipInfo.DrawSale(point);
+	shipInfo.DrawAttributes(point + Point(0, shipInfo.SaleHeight()));
 	
-	return info.SaleHeight();
+	return shipInfo.SaleHeight() + shipInfo.AttributesHeight();
 }
 
 
 
-bool ShipyardPanel::DrawItem(const string &name, const Point &point, int scrollY) const
+bool ShipyardPanel::HasItem(const string &name) const
 {
 	const Ship *ship = GameData::Ships().Get(name);
-	if(!shipyard.Has(ship))
-		return false;
-	
-	zones.emplace_back(point.X(), point.Y(), SHIP_SIZE / 2, SHIP_SIZE / 2, ship, scrollY);
+	return shipyard.Has(ship);
+}
+
+
+
+void ShipyardPanel::DrawItem(const string &name, const Point &point, int scrollY)
+{
+	const Ship *ship = GameData::Ships().Get(name);
+	zones.emplace_back(point, Point(SHIP_SIZE, SHIP_SIZE), ship, scrollY);
 	if(point.Y() + SHIP_SIZE / 2 < Screen::Top() || point.Y() - SHIP_SIZE / 2 > Screen::Bottom())
-		return true;
+		return;
 	
 	DrawShip(*ship, point, ship == selectedShip);
-	
-	return true;
 }
 
 
@@ -97,22 +130,21 @@ int ShipyardPanel::DividerOffset() const
 
 int ShipyardPanel::DetailWidth() const
 {
-	return 3 * ShipInfoDisplay::PanelWidth();
+	return 3 * shipInfo.PanelWidth();
 }
 
 
 
-
-int ShipyardPanel::DrawDetails(const Point &center) const
+int ShipyardPanel::DrawDetails(const Point &center)
 {
-	ShipInfoDisplay info(*selectedShip, player.GetSystem()->GetGovernment());
-	Point offset(info.PanelWidth(), 0.);
+	shipInfo.Update(*selectedShip, player.StockDepreciation(), player.GetDate().DaysSinceEpoch());
+	Point offset(shipInfo.PanelWidth(), 0.);
 	
-	info.DrawDescription(center - offset * 1.5);
-	info.DrawAttributes(center - offset * .5);
-	info.DrawOutfits(center + offset * .5);
+	shipInfo.DrawDescription(center - offset * 1.5);
+	shipInfo.DrawAttributes(center - offset * .5);
+	shipInfo.DrawOutfits(center + offset * .5);
 	
-	return info.MaximumHeight();
+	return shipInfo.MaximumHeight();
 }
 
 
@@ -122,10 +154,10 @@ bool ShipyardPanel::CanBuy() const
 	if(!selectedShip)
 		return false;
 	
-	int cost = selectedShip->Cost();
+	int64_t cost = player.StockDepreciation().Value(*selectedShip, day);
 	
 	// Check that the player has any necessary licenses.
-	int64_t licenseCost = LicenseCost();
+	int64_t licenseCost = LicenseCost(&selectedShip->Attributes());
 	if(licenseCost < 0)
 		return false;
 	cost += licenseCost;
@@ -137,7 +169,7 @@ bool ShipyardPanel::CanBuy() const
 
 void ShipyardPanel::Buy()
 {
-	int64_t licenseCost = LicenseCost();
+	int64_t licenseCost = LicenseCost(&selectedShip->Attributes());
 	if(licenseCost < 0)
 		return;
 	
@@ -149,8 +181,13 @@ void ShipyardPanel::Buy()
 			" If that is okay with you, go ahead and enter a name for your brand new ";
 	else
 		message = "Enter a name for your brand new ";
-	message += selectedShip->ModelName() + "!";
-	GetUI()->Push(new Dialog(this, &ShipyardPanel::BuyShip, message));
+	
+	if(modifier == 1)
+		message += selectedShip->ModelName() + "! (Or leave it blank to use a randomly chosen name.)";
+	else
+		message += selectedShip->PluralModelName() + "! (Or leave it blank to use randomly chosen names.)";
+	
+	GetUI()->Push(new NameDialog(this, &ShipyardPanel::BuyShip, message));
 }
 
 
@@ -160,10 +197,10 @@ void ShipyardPanel::FailBuy() const
 	if(!selectedShip)
 		return;
 	
-	int64_t cost = selectedShip->Cost();
+	int64_t cost = player.StockDepreciation().Value(*selectedShip, day);
 	
 	// Check that the player has any necessary licenses.
-	int64_t licenseCost = LicenseCost();
+	int64_t licenseCost = LicenseCost(&selectedShip->Attributes());
 	if(licenseCost < 0)
 	{
 		GetUI()->Push(new Dialog("Buying this ship requires a special license. "
@@ -175,7 +212,7 @@ void ShipyardPanel::FailBuy() const
 	if(player.Accounts().Credits() < cost)
 	{
 		for(const auto &it : player.Ships())
-			cost -= it->Cost();
+			cost -= player.FleetDepreciation().Value(*it, day);
 		if(player.Accounts().Credits() < cost)
 			GetUI()->Push(new Dialog("You do not have enough credits to buy this ship. "
 				"Consider checking if the bank will offer you a loan."));
@@ -201,9 +238,12 @@ bool ShipyardPanel::CanSell() const
 void ShipyardPanel::Sell()
 {
 	static const int MAX_LIST = 20;
+	static const int MAX_NAME_WIDTH = 250 - 30;
 	
 	int count = playerShips.size();
+	int initialCount = count;
 	string message = "Sell ";
+	const Font &font = FontSet::Get(14);
 	if(count == 1)
 		message += playerShip->Name();
 	else if(count <= MAX_LIST)
@@ -217,7 +257,7 @@ void ShipyardPanel::Sell()
 		else
 		{
 			while(count-- > 1)
-				message += ",\n" + (*it++)->Name();
+				message += ",\n" + font.TruncateMiddle((*it++)->Name(), MAX_NAME_WIDTH);
 			message += ",\nand ";
 		}
 		message += (*it)->Name();
@@ -225,12 +265,20 @@ void ShipyardPanel::Sell()
 	else
 	{
 		auto it = playerShips.begin();
-		for(int i = 0; i < MAX_LIST - 1; ++i)
-			message += (*it++)->Name() + ",\n";
+		message += (*it++)->Name() + ",\n";
+		for(int i = 1; i < MAX_LIST - 1; ++i)
+			message += font.TruncateMiddle((*it++)->Name(), MAX_NAME_WIDTH) + ",\n";
 		
 		message += "and " + Format::Number(count - (MAX_LIST - 1)) + " other ships";
 	}
-	message += "?";
+	// To allow calculating the sale price of all the ships in the list,
+	// temporarily copy into a shared_ptr vector:
+	vector<shared_ptr<Ship>> toSell;
+	for(const auto &it : playerShips)
+		toSell.push_back(it->shared_from_this());
+	int64_t total = player.FleetDepreciation().Value(toSell, day);
+	
+	message += ((initialCount > 2) ? "\nfor " : " for ") + Format::Number(total) + " credits?";
 	GetUI()->Push(new Dialog(this, &ShipyardPanel::SellShip, message));
 }
 
@@ -253,27 +301,26 @@ bool ShipyardPanel::CanSellMultiple() const
 
 void ShipyardPanel::BuyShip(const string &name)
 {
-	int64_t licenseCost = LicenseCost();
+	int64_t licenseCost = LicenseCost(&selectedShip->Attributes());
 	if(licenseCost)
 	{
 		player.Accounts().AddCredits(-licenseCost);
-		for(const string &name : selectedShip->Licenses())
-			if(player.GetCondition("license: " + name) <= 0)
-				player.Conditions()["license: " + name] = true;
+		for(const string &licenseName : selectedShip->Attributes().Licenses())
+			if(player.GetCondition("license: " + licenseName) <= 0)
+				player.Conditions()["license: " + licenseName] = true;
 	}
 	
-	string shipName = name;
-	if(shipName.empty())
-		shipName = player.FirstName() + "'s " + selectedShip->ModelName();
-	if(modifier > 1)
-		shipName += ' ';
-	
 	for(int i = 1; i <= modifier; ++i)
 	{
-		if(modifier > 1)
-			player.BuyShip(selectedShip, shipName + to_string(i));
-		else
-			player.BuyShip(selectedShip, shipName);
+		// If no name is given, choose a random name. Otherwise, if buying
+		// multiple ships, append a number to the given ship name.
+		string shipName = name;
+		if(name.empty())
+			shipName = GameData::Phrases().Get("civilian")->Get();
+		else if(modifier > 1)
+			shipName += " " + to_string(i);
+		
+		player.BuyShip(selectedShip, shipName);
 	}
 	
 	playerShip = &*player.Ships().back();
@@ -297,20 +344,5 @@ void ShipyardPanel::SellShip()
 		}
 	if(playerShip)
 		playerShips.insert(playerShip);
-}
-
-
-
-int64_t ShipyardPanel::LicenseCost() const
-{
-	int64_t cost = 0;
-	for(const string &name : selectedShip->Licenses())
-		if(player.GetCondition("license: " + name) <= 0)
-		{
-			const Outfit *outfit = GameData::Outfits().Get(name + " License");
-			if(!outfit->Cost())
-				return -1;
-			cost += outfit->Cost();
-		}
-	return cost;
+	player.UpdateCargoCapacities();
 }
