@@ -32,16 +32,17 @@ class Ship;
 // a government that is particularly repressive of independent pilots.)
 class ShipyardPanel : public ShopPanel {
 public:
-	ShipyardPanel(PlayerInfo &player);
+	explicit ShipyardPanel(PlayerInfo &player);
 	
 	
 protected:
 	virtual int TileSize() const override;
-	virtual int DrawPlayerShipInfo(const Point &point) const override;
-	virtual bool DrawItem(const std::string &name, const Point &point, int scrollY) const override;
+	virtual int DrawPlayerShipInfo(const Point &point) override;
+	virtual bool HasItem(const std::string &name) const override;
+	virtual void DrawItem(const std::string &name, const Point &point, int scrollY) override;
 	virtual int DividerOffset() const override;
 	virtual int DetailWidth() const override;
-	virtual int DrawDetails(const Point &center) const override;
+	virtual int DrawDetails(const Point &center) override;
 	virtual bool CanBuy() const override;
 	virtual void Buy() override;
 	virtual void FailBuy() const override;
@@ -54,7 +55,6 @@ protected:
 private:
 	void BuyShip(const std::string &name);
 	void SellShip();
-	int64_t LicenseCost() const;
 	
 	
 private:
